
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "credentialsDialog.h"

CredentialsDialog::CredentialsDialog(QString * username,
	QString * domain, QString * password,
	Farm *farm, QWidget *parent/* = 0*/)
    : QDialog(parent)
{
	setupUi(this);

	p_farm = farm;
	p_username = username;
	p_password = password;
	p_domain = domain;

	// Update interface
	farmNameLabel->setText(p_farm->getName());
	usernameLineEdit->setText(p_farm->getUsername());
	passwordLineEdit->setText(p_farm->getPassword());
        domainComboBox->addItems(p_farm->getDomainList());
        if (domainComboBox->count() == 0)
    	    domainComboBox->addItem(p_farm->getCurrentDomain());
        domainComboBox->setCurrentIndex(domainComboBox->findText(*p_domain));
        domainComboBox->setEditable(true);
    saveCheckBox->setChecked(p_farm->isCredentialsSaved());

	connect(okButton, SIGNAL(clicked()), SLOT(accept()));
	connect(cancelButton, SIGNAL(clicked()), SLOT(exit()));
}

CredentialsDialog::~CredentialsDialog()
{

}

void CredentialsDialog::accept()
{

	*p_username = usernameLineEdit->text();
	*p_password = passwordLineEdit->text();
	*p_domain = domainComboBox->currentText();

	if ((*p_username).isEmpty() || (*p_domain).isEmpty() || (*p_password).isEmpty())
	{
		QMessageBox::warning(this, "AppPortal", "Fields should not be empty");
		return;
	}

	// Save credentials if ti needs
	if (saveCheckBox->isChecked())
	{
		p_farm->setUsername(*p_username);
		p_farm->setPassword(*p_password);
		p_farm->setCurrentDomain(*p_domain);
		p_farm->setCredentialsSave(saveCheckBox->isChecked());
		//settings->setKerberosMode(!saveCheckBox->isChecked());
		QStringList domains;
		for (int i = 0; i < domainComboBox->count(); ++i)
		    domains << domainComboBox->itemText(i);
		p_farm->setDomainList(domains);
	}

	return QDialog::accept();
}

void CredentialsDialog::exit()
{
	QDialog::rejected();
	close();
}
