
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#ifndef CONNECTIVITYPAGE_H
#define CONNECTIVITYPAGE_H

#include <QtGui/QWidget>
#include <QtGui/QButtonGroup>
#include <QtGui/QInputDialog>
#include <QtGui/QMessageBox>
#include <QHttp>
#include "ui_connectivityPage.h"
#include "../manageConnectionPage.h"

class ConnectivityPage : public ManageConnectionPage,
	private Ui::ConnectivityPageClass
{
    Q_OBJECT

public:
    ConnectivityPage(Farm *farm, ManageConnectionDialog *parent = 0);
    ~ConnectivityPage();

    void saveFarmSettings();
    void loadFarmSettings();


protected:
	void updateExistingLocationsList();
	virtual void showEvent(QShowEvent *);
	virtual void hideEvent(QHideEvent *);

private slots:

    void currentLocationChanged(int);
	void addBroker();
	void removeBroker();
	void upBroker();
    void downBroker();
    void testBroker();
    void handleTestBroker( bool );
	void editItem(const QModelIndex &index);
	void setEnabledBrokerButtons(bool);
	void checkSelectedBroker();

	void renameLocation();
        void addLocation();
        void removeLocation();

private:
	void updateLocationName(const QString & text);
	void loadLocationSettings();
	void saveLocationSettings();
	QString showInputDialog(const QString & title, const QString & label, bool * ok, const QString & initText = "");

    QButtonGroup *buttonGroup;
    QHttp http;
};

#endif // CONNECTIVITYPAGE_H
