
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "credentialsPage.h"

CredentialsPage::CredentialsPage(Farm *farm,  bool pathThroughMode, ManageConnectionDialog *parent)
    : ManageConnectionPage(farm, parent)
{
    setupUi(this);

    connect(followingCredentialsRadioButton, SIGNAL(toggled(bool)),
    	this, SLOT(followCredentials(bool)));
    connect(domainComboBox->lineEdit(), SIGNAL(editingFinished()),
        this, SLOT(addDomain()));

    cachedCredentialsRadioButton->setEnabled(pathThroughMode);
}

CredentialsPage::~CredentialsPage()
{

}

void CredentialsPage::addDomain()
{
    QStringList domains;
    for (int i = 0; i < domainComboBox->count(); ++i)
        domains << domainComboBox->itemText(i);
    currentFarm->setDomainList(domains);
    currentFarm->setCurrentDomain(domainComboBox->currentText());
}

void CredentialsPage::followCredentials(bool bFollowCreds)
{
    userNameLineEdit->setEnabled(bFollowCreds);
    userNameLabel->setEnabled(bFollowCreds);
    userNameLineEdit->clear();

    passwordLineEdit->setEnabled(bFollowCreds);
    passwordLabel->setEnabled(bFollowCreds);
    passwordLineEdit->clear();

    domainComboBox->setEnabled(bFollowCreds);
    domainLabel->setEnabled(bFollowCreds);

    saveCheckBox->setChecked(false);
    saveCheckBox->setEnabled(bFollowCreds);
}

void CredentialsPage::kerberosCredentials(bool bKerberCreds)
{

}

void CredentialsPage::loadFarmSettings()
{
        // Farm doesn`t existing
        if (currentFarm == 0)
		return;

        // TODO
        //cachedCredentialsRadioButton->setChecked(currentFarm->isSsoEnabled());
        //kerberosCredentialsRadioButton->setChecked(currentFarm->isKerberosEnabled());
        //authenticationCheckBox->setChecked(currentFarm->isKerberosMode());

        userNameLineEdit->setText(currentFarm->getUsername());
        passwordLineEdit->setText(currentFarm->getPassword());
        domainComboBox->addItems(currentFarm->getDomainList());
        if (domainComboBox->count() == 0)
    	    domainComboBox->addItem(currentFarm->getCurrentDomain());
        saveCheckBox->setChecked(currentFarm->isCredentialsSaved());
        domainComboBox->setEditable(true);
        domainComboBox->setCurrentIndex(domainComboBox->findText(currentFarm->getCurrentDomain()));
}

void CredentialsPage::saveFarmSettings()
{
	// Farm doesn`t existing
        if (currentFarm == 0)
		return;

        currentFarm->setSsoEnable(cachedCredentialsRadioButton->isChecked());

        QString username = userNameLineEdit->text();
        QString password = passwordLineEdit->text();
        QString domain = domainComboBox->currentText();
        if (currentFarm->getUsername() != username ||
            currentFarm->getPassword() != password ||
            currentFarm->getCurrentDomain() != domain)
        {
            // force reconnection
            currentFarm->setNeedsReconnecting(true);
        }
        currentFarm->setUsername(username);
        currentFarm->setPassword(password);
        currentFarm->setCurrentDomain(domain);
        QStringList domains = currentFarm->getDomainList();
        if (!domains.contains(domain))
        {
            domains.append(domain);
            currentFarm->setDomainList(domains);
        }
        currentFarm->setCredentialsSave(saveCheckBox->isChecked());
}
