
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "displayPage.h"
#include "farm.h"
#include <QtDebug>

DisplayPage::DisplayPage(Farm *farm, ManageConnectionDialog *parent)
    : ManageConnectionPage(farm, parent)
{
	setupUi(this);

	displaySizeList << "640x480" << "800x480" << "800x600"
            << "1024x480" << "1024x600" << "1024x768"
            << "1280x768" << "1280x800" << "Full Screen";

        // Creating the list of picture filenames
        pixmapFileNames << ":/img/ColorSample256.png"
            << ":/img/ColorSample15bit.png"
            << ":/img/ColorSample16bit.png"
            << ":/img/ColorSample24True.png";

        displayDepthList << 8 << 15 << 16 << 24;

	connect(colorsDesktopComboBox, SIGNAL(currentIndexChanged(int)),
			SLOT(colorDepthChanged(int)));
        //connect(displayConnectionBarCheckBox, SIGNAL(toggled(bool)),
        //		SLOT(pinConnectionBarEnabled(bool)));
	connect(sizeDesktopSlider, SIGNAL(valueChanged(int)), this,
				SLOT(displaySizeChanged(int)));

        this->colorDepthChanged(0);
	this->displaySizeChanged(0);
}

DisplayPage::~DisplayPage()
{

}

void DisplayPage::colorDepthChanged(int index)
{
    // If index isn`t correct
    if (index < 0)
        return;

    QString currentPixmapName = pixmapFileNames.at(index);
    // No pixmap file for this index
    if (currentPixmapName.isEmpty())
        return;

    // Display correct pixamp on the page
    colorLabel->setPixmap(QPixmap(currentPixmapName));
}

void DisplayPage::saveFarmSettings()
{
	// Farm doesn`t existing
        if (currentFarm == 0)
		return;

        int index = 0;

        index = colorsDesktopComboBox->currentIndex();
        int colorDepth = displayDepthList.at(index);

	int width = 0;
	int height = 0;
	QString size = colorSizeLabel->text();
	index = size.indexOf("x");
	if (index > 0)
	{
		width = size.left(index).toInt();
		height = size.right(size.length() - index - 1).toInt();
	}

        currentFarm->setDesktopWidth(width);
        currentFarm->setDesktopHeight(height);
        currentFarm->setColorDepth(colorDepth);

        //currentFarm->setConnectionBar(displayConnectionBarCheckBox->isChecked());
        //currentFarm->setPinConnectionBar(pinConnectionBarCheckBox->isChecked());
        currentFarm->setSpanMonitors(spanMultipleMonitorsCheckBox->isChecked());
        //currentFarm->setSmartSizing(smartSizingCheckBox->isChecked());
        //currentFarm->setSeamlessMode(seamlesslyDesktopCheckBox->isChecked());
}

void DisplayPage::loadFarmSettings()
{
	// Farm doesn`t existing
        if (currentFarm == 0)
		return;

        int colorDepth = currentFarm->getColorDepth();
        int width = currentFarm->getDesktopWidth();
        int height = currentFarm->getDesktopHeight();
	
        int index = 0;

        index = displayDepthList.indexOf(colorDepth);
        if (index >= 0)
            colorsDesktopComboBox->setCurrentIndex(index);

        if (height == 0 || width == 0)
		sizeDesktopSlider->setValue(sizeDesktopSlider->maximum());
	else
	{
		QString size = QString::number(width) + "x" + QString::number(height);
                index = displaySizeList.indexOf(size);
		sizeDesktopSlider->setValue(index);
	}

        //displayConnectionBarCheckBox->setChecked(currentFarm->isConnectionBar());
        //pinConnectionBarCheckBox->setChecked(currentFarm->isPinConnectionBar());
        spanMultipleMonitorsCheckBox->setChecked(currentFarm->isSpanMonitors());
        //smartSizingCheckBox->setChecked(currentFarm->isSmartSizing());
        //seamlesslyDesktopCheckBox->setChecked(currentFarm->isSeamlessMode());
}


void DisplayPage::pinConnectionBarEnabled(bool pinEnabled)
{
    bool a;
//	pinConnectionBarCheckBox->setEnabled(pinEnabled);
//	pinConnectionBarCheckBox->setChecked(false);

    a = pinEnabled;
}

void DisplayPage::displaySizeChanged(int index)
{
	colorSizeLabel->setText(displaySizeList.value(index));
}




