
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "localResourcesPage.h"

LocalResourcesPage::LocalResourcesPage(Farm *farm, ManageConnectionDialog *parent)
    : ManageConnectionPage(farm, parent)
{
	setupUi(this);
    connect(daQualityComboBox, SIGNAL(currentIndexChanged(int)), SLOT(qualityChanged()));
}

LocalResourcesPage::~LocalResourcesPage()
{

}

void LocalResourcesPage::qualityChanged()
{
    const QString customQualString(tr("custom"));

    if (daQualityComboBox->currentText() == customQualString)
    {
        daCustomQualityLineEdit->setEnabled(true);
    }
    else
    {
        daCustomQualityLineEdit->setEnabled(false);
    }
}

void LocalResourcesPage::loadFarmSettings()
{
	// Farm doesn`t existing
        if (currentFarm == 0)
		return;

    QString soundMode = currentFarm->getSoundMode();
	int index = 2;
	if (soundMode == "local")
		index = 0;
	if (soundMode == "remote" )
		index = 1;
	soundComboBox->setCurrentIndex(index);

	//keyboardComboBox->setCurrentIndex(settings->value("KeyboardHook", 0).toInt());

        diskDriversCheckBox->setChecked(currentFarm->isRedirectDrives());
        printersCheckBox->setChecked(currentFarm->isRedirectPrinters());
        usbCheckBox->setChecked(currentFarm->isRedirectHandhelds());
        clipboardCheckBox->setChecked(currentFarm->isRedirectClipboard());
        serialPortsCheckBox->setChecked(currentFarm->isRedirectComPorts());
        //smartCardsCheckBox->setChecked(currentFarm->isRedirectSmartCards());
        //universalPrintersCheckBox->setChecked(currentFarm->isRedirectUPrinters());
        //microphoneCheckBox->setChecked(currentFarm->isRedirectMicrophone());

	QString daMode = currentFarm->getDAMode();
	if (daMode == "local")
        index = 0;
	else if (daMode == "remote")
        index = 1;
	else
        index = 2;

    daGroupBox->setChecked(currentFarm->isDownstreamAudioEnabled());
    daModeComboBox->setCurrentIndex(index);
    daQualityComboBox->setCurrentIndex(daQualityComboBox->findText(currentFarm->getDAQuality()));
    daBufferingLineEdit->setText(QString::number(currentFarm->getDABuffering()));
    daCustomQualityLineEdit->setText(QString::number(currentFarm->getDACustomQuality()));

    uaGroupBox->setChecked(currentFarm->isUpstreamAudioEnabled());
    uaModeComboBox->setCurrentIndex(uaModeComboBox->findText(currentFarm->getUAMode()));
    uaQualityLineEdit->setText(QString::number(currentFarm->getUAQuality()));
    uaComplexityLineEdit->setText(QString::number(currentFarm->getUAComplexity()));
    uaEnhancedComressionCheckBox->setChecked(currentFarm->getUAEnhancedCompression());

}

void LocalResourcesPage::saveFarmSettings()
{
	// Farm doesn`t existing
        if (currentFarm == 0)
		return;

	// sound
	int index = soundComboBox->currentIndex();
	QString soundMode;
	switch (index)
	{
	case 0:
		soundMode = "local";
		break;
	case 1:
		soundMode = "remote";
		break;
	default:
		soundMode = "off";
	}
    currentFarm->setSoundMode(soundMode);

	//settings->setValue("KeyboardHook", keyboardComboBox->currentIndex());

        currentFarm->setRedirectDrives(diskDriversCheckBox->isChecked());
        currentFarm->setRedirectPrinters(printersCheckBox->isChecked());
        currentFarm->setRedirectHandhelds(usbCheckBox->isChecked());
        currentFarm->setRedirectClipboard(clipboardCheckBox->isChecked());
        currentFarm->setRedirectComPorts(serialPortsCheckBox->isChecked());
        //currentFarm->setRedirectSmartCards(smartCardsCheckBox->isChecked());
        //currentFarm->setRedirectUPrinters(universalPrintersCheckBox->isChecked());
        //currentFarm->setRedirectMicrophone(microphoneCheckBox->isChecked());

    // downstream audio
    QString daMode, daQuality;
    switch(daModeComboBox->currentIndex()){
    case 0:
        daMode = "local"; break;
    case 1:
        daMode = "remote"; break;
    default:
        daMode = "donotplay";
    }
    currentFarm->setDownstreamAudioEnabled(daGroupBox->isChecked());
    currentFarm->setDAMode(daMode);
    currentFarm->setDAQuality(daQualityComboBox->currentText());
    currentFarm->setDABuffering(daBufferingLineEdit->text().toInt());
    currentFarm->setDACustomQuality(daCustomQualityLineEdit->text().toInt());

    currentFarm->setUpstreamAudioEnabled(uaGroupBox->isChecked());
    currentFarm->setUAMode(uaModeComboBox->currentText());
    currentFarm->setUAQuality(uaQualityLineEdit->text().toInt());
    currentFarm->setUAComplexity(uaComplexityLineEdit->text().toInt());
    currentFarm->setUAEnhancedCompression(uaEnhancedComressionCheckBox->isChecked());

    QString uaMode;
}
