
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "restartProgressDialog.h"
#include "mainWindow.h"

RestartProgressDialog *RestartProgressDialog::instance = NULL;

RestartProgressDialog::RestartProgressDialog(QProcess **proc, const QString &message, int timeout, int curAttemptNum)
    : process(proc),
    timer(NULL),
    timeoutTimer(timeout)
{
    const QString timeoutStrDefault(tr("Retrying in %1 seconds..."));
    const QString title(tr("Launching Application"));

    MainWindow *mainWindowInstance = MainWindow::instance();
    this->setParent(mainWindowInstance, Qt::Dialog);
    this->setWindowModality(Qt::WindowModal);
    this->setWindowTitle(title);

    messageLbl = new QLabel(message);
    timerLbl = new QLabel();
    QPushButton *cancelBtn = new QPushButton(tr("Cancel"));
    QHBoxLayout *horLayout = new QHBoxLayout();
    horLayout->addWidget(timerLbl);
    horLayout->addItem(new QSpacerItem(40, 0));
    horLayout->addWidget(cancelBtn);

    QVBoxLayout *vertLayout = new QVBoxLayout();
    vertLayout->addWidget(messageLbl);
    vertLayout->addItem(horLayout);
    this->setLayout(vertLayout);

    QString timeoutStr = timeoutStrDefault.arg(timeout);
    timerLbl->setText(timeoutStr);

    connect(this, SIGNAL(rejected()), this, SLOT(onRetryingCancel()));
    connect(cancelBtn, SIGNAL(clicked()), this, SLOT(onRetryingCancel()));
}

RestartProgressDialog* RestartProgressDialog::getInstance(QProcess **proc, const QString &message, int timeout, int curAttemptNum)
{
    if (instance == NULL)
    {
        instance = new RestartProgressDialog(proc, message, timeout, curAttemptNum);
    }
    else
    {
        reinit(instance, proc, message, timeout, curAttemptNum);
    }
    return instance;
}

void RestartProgressDialog::reinit(RestartProgressDialog *dlg, QProcess **proc, const QString &message, int timeout, int curAttemptNum)
{
    const QString timeoutStrDefault(tr("Retrying in %1 seconds..."));

    dlg->process = proc;
    if (dlg->timer != NULL)
    {
        dlg->timer->stop();
        dlg->timer = NULL;
    }

    dlg->timeoutTimer = timeout;
    dlg->messageLbl->setText(message);
    QString timeoutStr = timeoutStrDefault.arg(timeout);
    dlg->timerLbl->setText(timeoutStr);
}

void RestartProgressDialog::show()
{
    if (timer != NULL)
    {
        timer->stop();
    }

    this->setVisible(true);

    restartTimer();
}

void RestartProgressDialog::restartTimer()
{
    const int timerStep = 1000; //in millis

    timer = new QTimer();
    connect(timer, SIGNAL(timeout()), this, SLOT(updateOnPowerTimeout()));
    timer->setInterval(timerStep);
    timer->start();
}

void RestartProgressDialog::updateOnPowerTimeout()
{
    if (timeoutTimer <= 0)
    {
        timer->stop();
        this->setVisible(false);
        return;
    }
    QString timeoutStr = QString(tr("Retrying in %1 seconds...")).arg(--timeoutTimer);
    timerLbl->setText(timeoutStr);
    this->repaint();
}

void RestartProgressDialog::onRetryingCancel()
{
    if (process != NULL && (*process) != NULL)
    {
        (*process)->terminate();
    }

    if (timer != NULL)
    {
        timer->stop();
    }
    this->setVisible(false);
}
