
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "treeMenuWidget.h"
#include "sublib/icons.h"

#include <QtDebug>

TreeMenuWidget::TreeMenuWidget (QWidget *parent)
  :QTreeWidget (parent)
{
    createTreeMenuWidget(this);

    connect(this, SIGNAL(itemActivated(QTreeWidgetItem *, int)),
            SLOT(applicationActivated(QTreeWidgetItem *, int)));
}

void TreeMenuWidget::createTreeMenuWidget (QTreeWidget *treeMenu)
{
  treeMenu->setHeaderHidden(true);
  treeMenu->setMinimumWidth(200);

  this->clear();
}

void TreeMenuWidget::updateApplicationsView(ApplicationList apps)
{
    this->clear();
    if (apps.isEmpty())
        return;

    //qDebug() << apps.names();

    foreach (Application * app, apps)
    {
        // Folder of application
        QStringList appFolderPath = app->folder().split("//", QString::SkipEmptyParts);

        QTreeWidgetItem *lastFolderItem = 0;
        foreach (QString pathKey, appFolderPath)
	{
            QList<QTreeWidgetItem *> existKeys;

            if (lastFolderItem == 0)
            {
                existKeys = this->findItems(pathKey, Qt::MatchExactly);
            }
            else
            {
                for (int i = 0; i < lastFolderItem->childCount(); i++)
                    if (lastFolderItem->child(i)->text(0) == pathKey)
                    existKeys.append(lastFolderItem->child(i));
            }

            if (existKeys.isEmpty())
            {
                QTreeWidgetItem *folderItem = new QTreeWidgetItem(this);
                folderItem->setText(0, pathKey);
                folderItem->setIcon(0, QIcon(":/img/folder_icon.png"));
                lastFolderItem = folderItem;
            }
            else
                lastFolderItem = existKeys.first();
        }

        // Adding application record
        QTreeWidgetItem *newApplicationItem = new QTreeWidgetItem(lastFolderItem);

        QPixmap *appSmallPixmap =  app->smallIcon();
        if (appSmallPixmap != 0)
            newApplicationItem->setIcon(0, QIcon(*appSmallPixmap));

        QString appName = app->name();
        newApplicationItem->setText(0, appName);

        if (!app->isEnabled())
        {
            newApplicationItem->setFlags(!Qt::ItemIsSelectable | !Qt::ItemIsEnabled);
        }
    }
}

void TreeMenuWidget::applicationActivated(QTreeWidgetItem *item, int column)
{
    if (item->childCount() != 0 || item == rootItem || ((item->flags() & Qt::ItemIsEnabled) == 0))
        return;

    QString currentApplicationName = item->text(column);
    emit executeApplication(currentApplicationName);

}

void TreeMenuWidget::clear()
{
    QTreeWidget::clear();
}
