
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


/*
 *
 * Copyright (C) 2007-2009 SimplyCore LLC
 * All rights reserved.
 * This file is part of Linux USB IP project.
 * Distributed under Non-disclosure agreement.
 *
 */

#ifndef _USBD_FILTER_H_
#define _USBD_FILTER_H_

#ifndef __KERNEL__
#include <syslog.h>
#endif

#include "nettypes.h"

static int usbd_is_autosharing_allowed(
	const uint16 vid, 
	const uint16 pid, 
	const uint16 rev,
	const uint8 deviceClass, 
	const uint8 deviceSubClass, 
	const uint8 deviceProtocol, 
	const void * reserved)
{
	int rc;

    // Dump to debug
#ifdef __KERNEL__
    TRACE_CRITICAL( "Autoshare check for device.  vid=0x%04X, pid=0x%04X, rev=0x%04X, class=%i, subclass=%i, protocol=%i.\n", vid, pid, rev, deviceClass, deviceSubClass, deviceProtocol );
#else
    syslog( LOG_INFO, "Autoshare check for device.  vid=0x%04X, pid=0x%04X, rev=0x%04X, class=%i, subclass=%i, protocol=%i.\n", vid, pid, rev, deviceClass, deviceSubClass, deviceProtocol );
#endif
	
	rc = 1; // Allow by default
	
	// Never share USB Keyboards, class:3 subclass:0 protocol:1
	// Never share USB Mice     , class:3 subclass:0 protocol:2
	//
	// Check for cetain HID class devices to block
	if ( deviceClass == 3 )
		{
		// Block Mice and Keyboards
		if (( deviceProtocol == 1 ) || ( deviceProtocol == 2 )) rc = 0;
		}

    // Dump to debug
#ifdef __KERNEL__
    TRACE_CRITICAL( "Autoshare result is %i.\n", rc );
#else
    syslog( LOG_INFO, "Autoshare result is %i.\n", rc );
#endif
		
	return rc;
}

#endif // _USBD_FILTER_H_
