
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


/*
 *
 * Copyright (C) 2007-2009 SimplyCore LLC
 * All rights reserved.
 * This file is part of Linux USB IP project.
 * Distributed under Non-disclosure agreement.
 *
 */

#ifndef _PUBSTRCT_
#define _PUBSTRCT_

#include <set>
#include <string>
#include <string.h>

#pragma pack(push, 1)
typedef struct _taghwid {
	unsigned short idVendor;
	unsigned short idProduct;
	unsigned short bcdDevice;
} __attribute__((packed)) HWID, *PHWID;

#pragma pack(pop)

typedef struct _tagiface {
	unsigned char Class;
	unsigned char SubClass;
	unsigned char Protocol;
} __attribute__((packed)) ZIFACE, *PZIFACE;

#ifdef __cplusplus

	inline bool operator < (const struct _taghwid& usbhwid1,
				 const struct _taghwid& usbhwid2)
	{
		return (memcmp(&usbhwid1, &usbhwid2, sizeof(usbhwid1)) < 0);
	}

	inline bool operator == (const struct _taghwid& usbhwid1,
				 const struct _taghwid& usbhwid2)
	{
		return (memcmp(&usbhwid1, &usbhwid2, sizeof(usbhwid1)) == 0);
	}

	inline bool operator != (const struct _taghwid& usbhwid1,
				 const struct _taghwid& usbhwid2)
	{
		return (!memcmp(&usbhwid1, &usbhwid2, sizeof(usbhwid1)) == 0);
	}

#endif

//    .
typedef struct _md5handle{
	char		md5[16];
	unsigned long	nHandle;
} md5handle, *pmd5handle;

//    (hwid) USB 
typedef struct _tagdevprop {
	char	locationinfo[256];
	char	devdescription[256];
	char	hardwareid[256];
	char	compatibleid[256];
}DEVPROP, *PDEVPROP;

//       
typedef struct _tagusbdevs {

	HWID			hwid;
	unsigned long	nDeviceProperties;
	std::string		busid;
	std::string		nickname;
	std::string		locinfo;
	std::string		devdesc;

} TEMPUSBINFO, *PTEMPUSBINFO;

//    USB 
typedef struct _tagusbdevice {
	int				status;
	int				major;
	int				minor;
	int 			nDeviceFlags;
	char			nDeviceSpeed;
	void			*pclass;
	unsigned long	ip_client;
	unsigned long	clienthandle;
	HWID			hwid;
	ZIFACE			ziface;
	std::string		busid;
	std::string		serialnumber;
	std::string		nickname;
}USBDEV, *PUSBDEV;

//      .
typedef struct _taggrpdevs {
	int					devprop_valid;
	DEVPROP				devprop;
	std::set <int>		handles;
} GRPDEVS, *PGRPDEVS;

//   USB     
typedef struct _tagusbdevstate {
	int hDevice;
	int hServer;
	HWID hwid;
	ZIFACE ziface;
	int	nDeviceStatus;
	int nDeviceFlags;
	char nDeviceSpeed;
	int ipclient;
	int nInactiveTimeout;
	std::string busid;
	std::string	locationinformation;
	std::string devicedescription;
	std::string nickname;
	std::string serialnumber;
} USBDEVSTATE, *PUSBDEVSTATE;

//     
typedef struct _tagusbserverstate {
	int				hServer;
	int				nServerStatus;
	int				nPlatformId;
	std::string		sHostName;
	unsigned short	tcp_port;
	unsigned long	ulProperties;
} USBSERVERSTATE, *PUSBSERVERSTATE;

#endif // _PUBSTRCT_
