
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


/*
 *  tusbd/waitable_queue.h
 *
 *  Copyright (C) 2007-2008 IncentivesPro
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#ifndef _WAITABLE_QUEUE_H
#define _WAITABLE_QUEUE_H

struct waitable_queue
{
	struct list_head entrylist;	// list of entries
	spinlock_t lock;
	wait_queue_head_t waitqueue;
};

struct waitable_queue_entry
{
	struct list_head entry;
	unsigned long datasize;
	u8 data[1];
};

void usbd_wq_init(struct waitable_queue *pqueue);
void usbd_wq_deinit(struct waitable_queue *pqueue);

struct waitable_queue_entry *usbd_wq_alloc_entry(unsigned long datasize, int mem_flags);
void usbd_wq_free_entry(struct waitable_queue_entry *pentry);

// void usbd_wq_lock(struct waitable_queue *pqueue);
#define usbd_wq_lock(__pqueue, __flags) spin_lock_irqsave(&(__pqueue)->lock,__flags)
// void usbd_wq_unlock(struct waitable_queue *pqueue);
#define usbd_wq_unlock(__pqueue, __flags) spin_unlock_irqrestore(&(__pqueue)->lock,__flags)

#define usbd_wq_add_tail_locked(__pqueue, __pentry) \
		do { \
			unsigned long __flags; \
			usbd_wq_lock((__pqueue), __flags); \
			usbd_wq_add_tail((__pqueue), (__pentry)); \
			usbd_wq_unlock((__pqueue), (__flags)); \
		}while(0)

#define usbd_wq_remove_entry_locked(__pqueue, __pentry) \
		do { \
			unsigned long __flags; \
			usbd_wq_lock((__pqueue), (__flags)); \
			usbd_wq_remove_entry((__pentry)); \
			usbd_wq_unlock((__pqueue), __flags); \
		}while(0)

// the following functions should be synchronized with usbd_wq_lock/usbd_wq_unlock
int usbd_wq_is_empty(struct waitable_queue *pqueue);
int usbd_wq_is_queued(struct waitable_queue_entry *pentry);
void usbd_wq_add_tail(struct waitable_queue *pqueue, struct waitable_queue_entry *pentry);
struct waitable_queue_entry *usbd_wq_remove_head(struct waitable_queue *pqueue);
struct waitable_queue_entry *usbd_wq_get_first(struct waitable_queue *pqueue);
struct waitable_queue_entry *usbd_wq_get_next(struct waitable_queue *pqueue, struct waitable_queue_entry *pentry);
void usbd_wq_remove_entry(struct waitable_queue_entry *pentry);
void usbd_wq_clear(struct waitable_queue *pqueue);
void usbd_wq_wake_up(struct waitable_queue *pqueue);


#endif // _WAITABLE_QUEUE_H
