
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "dialogpages.h"


ClientOptionsPage::ClientOptionsPage(ConnectionDialog *parent)
	: ConnectionDialogPage(parent)
{
	setTitle(tr("Client options"), tr("Specify client options for the connection."));

	clientNameLabel = new QLabel("Client name");
	keyboardLayoutLabel = new QLabel("Keyboard layout");
	noteLabel = new QLabel("Note: some settings may be ignored if server-side "
						   "virtual channel policies are in effect.");
	noteLabel->setWordWrap(true);
	clientNameLineEdit = new QLineEdit;
    clientNameLineEdit->setMaximumHeight(22);
    clientNameLineEdit->setMaximumWidth(150);
	soundLineEdit = new QLineEdit;
	soundLineEdit->setVisible(false);
	keyboardLayoutComboBox = new QComboBox;
	keyboardLayoutComboBox->addItem("Local keyboard");
    keyboardLayoutComboBox->setMaximumHeight(25);
    keyboardLayoutComboBox->setMaximumWidth(150);
	soundCheckBox = new QCheckBox("Enable sound");
	sendingMouseCheckBox = new QCheckBox("Disable sending mouse movements\nto server");
    enableMmrCheckBox = new QCheckBox("Enable Multimedia Redirection");
    enableFmmrCheckBox = new QCheckBox("Enable Flash redirection");
    enableUsbrdrCheckBox = new QCheckBox("Enable USB Device Redirection");
    enableGaCheckBox = new QCheckBox("Enable Graphics Acceleration");

    // bi-di audio stuff
    downstreamAudioGroupBox = new QGroupBox(tr("Downstream Audio"));
    downstreamAudioGroupBox->setCheckable(true);
//    downstreamAudioGroupBox->setContentsMargins(0, 0, 0, 5);
    daModeComboBox = new QComboBox;
    daModeComboBox->addItem("Local");
    daModeComboBox->addItem("Remote");
    daModeComboBox->addItem("Do not play");
    daModeComboBox->setMaximumHeight(25);
    daModeComboBox->setMaximumWidth(120);
    daQualiltyComboBox = new QComboBox;
    daQualiltyComboBox->addItem("radio");
    daQualiltyComboBox->addItem("telephony");
    daQualiltyComboBox->addItem("tp");
    daQualiltyComboBox->addItem("cd");
    daQualiltyComboBox->addItem("custom");
    daQualiltyComboBox->setMaximumHeight(25);
    daQualiltyComboBox->setMaximumWidth(120);
    daBufferingLineEdit = new QLineEdit;
    daBufferingLineEdit->setMaximumHeight(22);
    daBufferingLineEdit->setMaximumWidth(120);
    daBufferingLineEdit->setText("0");
    daCustomQualityLineEdit = new QLineEdit;
    daCustomQualityLineEdit->setInputMask("9000000");
    daCustomQualityLineEdit->setMaximumHeight(22);
    daCustomQualityLineEdit->setMaximumWidth(120);
    daCustomQualityLineEdit->setText("0");
    QLabel *daLabel1 = new QLabel("Mode");
    QLabel *daLabel2 = new QLabel("Quality");
    QLabel *daLabel3 = new QLabel("Buffering");
    QLabel *daLabel4 = new QLabel("Custom quality");

    upstreamAudioGroupBox = new QGroupBox(tr("Upstream Audio"));
    upstreamAudioGroupBox->setCheckable(true);
    uaModeComboBox = new QComboBox;
    uaModeComboBox->addItem("wave");
    uaModeComboBox->addItem("speex");
    uaModeComboBox->setMaximumHeight(25);
    uaModeComboBox->setMaximumWidth(120);
    uaQualityLineEdit = new QLineEdit();
    uaQualityLineEdit->setMaximumHeight(22);
    uaQualityLineEdit->setMaximumWidth(120);
    uaQualityLineEdit->setInputMask("9000000");
    uaQualityLineEdit->setText("0");
    uaComplexityLineEdit = new QLineEdit();
    uaComplexityLineEdit->setMaximumHeight(22);
    uaComplexityLineEdit->setMaximumWidth(120);
    uaComplexityLineEdit->setInputMask("9000000");
    uaComplexityLineEdit->setText("0");
    QLabel *uaLabel1 = new QLabel("Mode");
    QLabel *uaLabel2 = new QLabel("Quality");
    QLabel *uaLabel3 = new QLabel("Complexity");
    uaEnhancedCheckBox = new QCheckBox("Enable enhanced compression");

    QStringList rdesktopLayouts;
    rdesktopLayouts << "ar" << "common" << "convert-map" << "cs" << "da" << "de"
                    << "de-ch" << "en-dv" << "en-gb" << "en-us" << "es" << "et"
                    << "fi" << "fo" << "fr" << "fr-be" << "fr-ca" << "fr-ch"
                    << "he" << "hr" << "hu" << "is" << "it" << "ja" << "ko"
                    << "lt" << "lv" << "mk" << "modifiers" << "nl" << "nl-be"
                    << "no" << "pl" << "pt" << "pt-br" << "ru" << "sl" << "sv"
                    << "th" << "tr";
    int i = 0;
    for (; i < rdesktopLayouts.size(); i++)
    {
    	keyboardLayoutComboBox->addItem(rdesktopLayouts.at(i));
    }
    keyboardLayoutComboBox->setCurrentIndex(0);

	registerField("clientName", clientNameLineEdit);
	registerField("keyboardLayout", keyboardLayoutComboBox);
	registerField("sound", soundLineEdit);
	registerField("multimediaredirection", enableMmrCheckBox);
	registerField("flashredirection", enableFmmrCheckBox);
	registerField("redirecthandhelds", enableUsbrdrCheckBox);
	registerField("sendingMouse", sendingMouseCheckBox);
	registerField("bitmapacceleration", enableGaCheckBox);
	registerField("downstreamAudioEnabled", downstreamAudioGroupBox);
	registerField("daMode", daModeComboBox);
	registerField("daQuality", daQualiltyComboBox);
	registerField("daBuffering", daBufferingLineEdit);
	registerField("daCustomQuality", daCustomQualityLineEdit);
	registerField("upstreamAudioEnabled", upstreamAudioGroupBox);
	registerField("uaMode", uaModeComboBox);
	registerField("uaQuality", uaQualityLineEdit);
	registerField("uaComplexity", uaComplexityLineEdit);
	registerField("uaEnhancedCompression", uaEnhancedCheckBox);

	QIntValidator * validator = new QIntValidator(0);
    validator->setBottom(0);
    daBufferingLineEdit->setValidator(validator);

    connect(soundCheckBox, SIGNAL(toggled(bool)),
            this, SLOT(enableSound(bool)));
    connect(soundLineEdit, SIGNAL(textChanged(const QString &)),
	    this, SLOT(setSoundCheckBox(const QString &)));

	QGridLayout *gridLayout1 = new QGridLayout;
	gridLayout1->addWidget(clientNameLabel, 0, 0);
	gridLayout1->addWidget(keyboardLayoutLabel, 1, 0);
	gridLayout1->addWidget(clientNameLineEdit, 0, 1);
	gridLayout1->addWidget(keyboardLayoutComboBox, 1, 1);
	gridLayout1->setHorizontalSpacing(20);
	gridLayout1->setContentsMargins(0, 0, 0, 15);

    QGridLayout *daLayout = new QGridLayout;
    daLayout->addWidget(daLabel1, 0, 0);
    daLayout->addWidget(daLabel2, 1, 0);
    daLayout->addWidget(daLabel3, 2, 0);
    daLayout->addWidget(daLabel4, 3, 0);
    daLayout->addWidget(daModeComboBox, 0, 1);
    daLayout->addWidget(daQualiltyComboBox, 1, 1);
    daLayout->addWidget(daBufferingLineEdit, 2, 1);
    daLayout->addWidget(daCustomQualityLineEdit, 3, 1);
    QGridLayout *uaGridLayout = new QGridLayout;
    uaGridLayout->addWidget(uaLabel1, 0, 0);
    uaGridLayout->addWidget(uaLabel2, 1, 0);
    uaGridLayout->addWidget(uaLabel3, 2, 0);
    uaGridLayout->addWidget(uaModeComboBox, 0, 1);
    uaGridLayout->addWidget(uaQualityLineEdit, 1, 1);
    uaGridLayout->addWidget(uaComplexityLineEdit, 2, 1);
    QVBoxLayout *uaLayout = new QVBoxLayout;
    uaLayout->addLayout(uaGridLayout);
    uaLayout->addWidget(uaEnhancedCheckBox);

    downstreamAudioGroupBox->setLayout(daLayout);
    upstreamAudioGroupBox->setLayout(uaLayout);

	QVBoxLayout *noteLayout = new QVBoxLayout;
	noteLayout->addWidget(noteLabel);
	noteLayout->setContentsMargins(5, 0, 0, 0);

	QVBoxLayout *vlLayout = new QVBoxLayout;
	vlLayout->addLayout(gridLayout1);
	vlLayout->addWidget(soundCheckBox);
	vlLayout->addWidget(sendingMouseCheckBox);
	vlLayout->addWidget(enableMmrCheckBox);
	vlLayout->addWidget(enableFmmrCheckBox);
	vlLayout->addWidget(enableUsbrdrCheckBox);
	vlLayout->addWidget(enableGaCheckBox);
	vlLayout->addLayout(noteLayout);
	vlLayout->setSpacing(2);

	QVBoxLayout *vrLayout = new QVBoxLayout;
	vrLayout->addWidget(downstreamAudioGroupBox);
	vrLayout->addWidget(upstreamAudioGroupBox);

	QHBoxLayout *hLayout = new QHBoxLayout;
	hLayout->addLayout(vlLayout);
	hLayout->addLayout(vrLayout);

	QVBoxLayout *layout = new QVBoxLayout;
	layout->addLayout(titleLayout);
	layout->addLayout(hLayout);
	layout->setAlignment(Qt::AlignTop);
	setLayout(layout);
}

void ClientOptionsPage::enableSound(bool enable)
{
    if (enable)
    {
        soundLineEdit->setText("local");
    }
    else
    {
        soundLineEdit->setText("off");
    }
}

void ClientOptionsPage::setSoundCheckBox(const QString &text)
{
    if (text == "local" && !soundCheckBox->isChecked())
    {
	soundCheckBox->setChecked(true);
    }
    if (text != "local" && soundCheckBox->isChecked())
    {
	soundCheckBox->setChecked(false);
    }
}
