
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "dialogpages.h"


ConnectionPage::ConnectionPage(ConnectionDialog *parent)
    : ConnectionDialogPage(parent)
{
    setTitle(tr("Connectivity"),
    		 tr("Specify the broker address(es) and connection options."));

    brokerOptionsGroupBox = new QGroupBox(tr("Connection Broker Settings"));
    brokersLabel = new QLabel(tr("Connection\nBrokers:"));
    brokersList = new QListWidget;
    brokerLineEdit = new QLineEdit;
    brokerLineEdit->setVisible(false);
	MyPushButtonCreator creator;
    addBrokerButton = creator.createButton("Add broker", 120);
    removeBrokerButton = creator.createButton("Remove broker", 120);
    downBrokerButton = creator.createButton("\\/", 30);
    upBrokerButton = creator.createButton("/\\", 30);
    QVBoxLayout *brokerLayout = new QVBoxLayout;
    brokerLayout->addWidget(addBrokerButton);
    brokerLayout->addWidget(removeBrokerButton);
    brokerLayout->setContentsMargins(0,0,0,20);
    brokerLayout->setAlignment(Qt::AlignTop);
    brokerLayout->setSpacing(0);
    QVBoxLayout *brokerCtrlLayout = new QVBoxLayout;
    brokerCtrlLayout->addWidget(upBrokerButton);
    brokerCtrlLayout->addWidget(downBrokerButton);
    brokerCtrlLayout->setContentsMargins(0,0,0,20);
    brokerCtrlLayout->setAlignment(Qt::AlignTop);
    brokerCtrlLayout->setSpacing(0);

    protocolLabel = new QLabel(tr("Protocol:"));
    protocolComboBox = new QComboBox;
    protocolComboBox->addItem("HTTPS");
    protocolComboBox->addItem("HTTP");
    portLabel = new QLabel(tr("TCP Port:"));
    portLineEdit = new QLineEdit;
    portValidator = new QIntValidator(1, 65535, this);
    portLineEdit->setValidator(portValidator);
    connectionOptionsGroupBox = new QGroupBox(tr("Connectivity"));
    natEnabledCheckBox = new QCheckBox(tr("Enable NAT support for firewall traversal (Terminal Server only)"));
    sslEnabledCheckBox = new QCheckBox(tr("Enable RDP over SSL\\TLS"));
    sslGatewayLabel = new QLabel(tr("   SSL Gateway FQDN:"));
    sslGatewayLineEdit = new QLineEdit;
    sslGatewayLabel->setEnabled(false);
    sslGatewayLineEdit->setEnabled(false);
    waitLabel = new QLabel(tr("Retry interval:"));
    waitSpinBox = new QSpinBox;
    waitSpinBox->setRange (0, 30);
    waitSpinBox->setSingleStep (5);
    waitSpinBox->setValue (5);
    retryLabel = new QLabel(tr("Number of retries:"));
    retrySpinBox = new QSpinBox;
    retrySpinBox->setRange (0, 20);
    retrySpinBox->setSingleStep (5);
    retrySpinBox->setValue (20);

    connect(addBrokerButton, SIGNAL(clicked()), this, SLOT(addBroker()));
    connect(removeBrokerButton, SIGNAL(clicked()), this, SLOT(removeBroker()));
    connect(sslEnabledCheckBox, SIGNAL(toggled(bool)),
            this, SLOT(setSslEnabled(bool)));
    connect(brokersList, SIGNAL(doubleClicked ( const QModelIndex &)),
            this, SLOT(editItem(const QModelIndex &)));
    connect(brokerLineEdit, SIGNAL(textChanged(const QString&)),
    		this, SLOT(setBrokerList(const QString&)));
    connect(upBrokerButton, SIGNAL(clicked()), this, SLOT(upBroker()));
    connect(downBrokerButton, SIGNAL(clicked()), this, SLOT(downBroker()));

	registerField("llmservers*", brokerLineEdit);
    registerField("llmtcpport*", portLineEdit);
    registerField("llmprotocol", protocolComboBox);
    registerField("llmenablenat", natEnabledCheckBox);
    registerField("enableSSL", sslEnabledCheckBox);
    registerField("sslgateway*", sslGatewayLineEdit);
    registerField("wait", waitSpinBox);
    registerField("retry", retrySpinBox);

    QGridLayout *groupBox1Layout = new QGridLayout;
    groupBox1Layout->addWidget(brokersLabel, 0, 0);
    groupBox1Layout->addWidget(brokersList, 0, 1);
    groupBox1Layout->addLayout(brokerCtrlLayout, 0,2);
    groupBox1Layout->addLayout(brokerLayout, 0, 3);
    groupBox1Layout->addWidget(protocolLabel, 1, 0);
    groupBox1Layout->addWidget(protocolComboBox, 1, 1);
    groupBox1Layout->addWidget(portLabel, 1, 2);
    groupBox1Layout->addWidget(portLineEdit, 1, 3);
    brokerOptionsGroupBox->setLayout(groupBox1Layout);

    QVBoxLayout *groupBox2Layout = new QVBoxLayout;
    QHBoxLayout *wLayout = new QHBoxLayout;
    wLayout->addWidget(waitLabel);
    wLayout->addWidget(waitSpinBox);
    wLayout->setAlignment(Qt::AlignLeft);
    QHBoxLayout *rLayout = new QHBoxLayout;
    rLayout->addWidget(retryLabel);
    rLayout->addWidget(retrySpinBox);
    rLayout->setAlignment(Qt::AlignLeft);
    QGridLayout *wrLayout = new QGridLayout;
    wrLayout->addLayout(wLayout, 0, 0);
//    wrLayout->addWidget(retryLabel);
//    wrLayout->addWidget(retrySpinBox);
    wrLayout->addLayout(rLayout, 0, 1);
    wrLayout->setHorizontalSpacing(50);
    groupBox2Layout->addLayout(wrLayout);
    groupBox2Layout->addWidget(natEnabledCheckBox);
    groupBox2Layout->addWidget(sslEnabledCheckBox);
    QHBoxLayout *sslLayout = new QHBoxLayout;
    sslLayout->addWidget(sslGatewayLabel);
    sslLayout->addWidget(sslGatewayLineEdit);
    groupBox2Layout->addLayout(sslLayout);
    connectionOptionsGroupBox->setLayout(groupBox2Layout);

    QVBoxLayout *layout = new QVBoxLayout;
    layout->addLayout(titleLayout);
    layout->addWidget(brokerOptionsGroupBox);
    layout->addWidget(connectionOptionsGroupBox);
    layout->setAlignment(Qt::AlignTop);
    setLayout(layout);

    checkBrokerListState();
}

//----------------------------------------------------------------------------
void ConnectionPage::upBroker()
{
    if (brokersList->currentItem() != NULL)
    {
        int currentIndex = brokersList->currentRow();
        if (currentIndex > 0)
        {
            QListWidgetItem *tempItem = brokersList->takeItem(currentIndex);
            brokersList->insertItem(currentIndex - 1, tempItem->text());
            brokersList->setCurrentRow(currentIndex - 1);
        }
        brokersList->setFocus();

        updateBrokerLineEdit();
    }
}

//----------------------------------------------------------------------------
void ConnectionPage::downBroker()
{
    if (brokersList->currentItem() != NULL)
    {
        int currentIndex = brokersList->currentRow();
        if (currentIndex < brokersList->count() - 1)
        {
            QListWidgetItem *tempItem = brokersList->takeItem(currentIndex);
            brokersList->insertItem(currentIndex + 1, tempItem->text());
            brokersList->setCurrentRow(currentIndex + 1);
        }
        brokersList->setFocus();

        updateBrokerLineEdit();
    }
}

//----------------------------------------------------------------------------
void ConnectionPage::updateBrokerLineEdit()
{
    brokerLineEdit->setText(brokersList->item(0)->text());
    for(int i = 1; i < brokersList->count(); ++i)
    {
        brokerLineEdit->setText(brokerLineEdit->text() + "," + brokersList->item(i)->text());
    }
}

//----------------------------------------------------------------------------
void ConnectionPage::addBroker()
{
    bool ok;
    QString text = QInputDialog::getText(this, "Add Connection Broker",
                                            "New Connection Broker:                       ",
                                             QLineEdit::Normal, QString(), &ok);
    if (ok && !text.isEmpty()) {
    	text.remove("http://");
    	text.remove("https://");
        if (brokersList->findItems(text, Qt::MatchExactly).isEmpty())
    	{
      	    brokersList->addItem(text);
            if (brokerLineEdit->text() == "")
    	    {
                brokerLineEdit->setText(text);
            }
    	    else
        	{
                brokerLineEdit->setText(brokerLineEdit->text() + "," + text);
    	    }
        }
        else
        {
    	    QMessageBox::critical(this, "Brokers list", "The list already contains this item");
        }
    }

    checkBrokerListState();
}

//----------------------------------------------------------------------------
void ConnectionPage::removeBroker()
{
    if (brokersList->currentItem() != NULL)
    {
        if (QMessageBox::question(this, this->windowTitle(),
                                  tr("Are you sure you want to delete the broker [") + brokersList->currentItem()->text() + "]?",
                                  QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
        {
            return;
        }


        brokerLineEdit->setText(brokerLineEdit->text().remove(brokersList->currentItem()->text()));
        if (brokerLineEdit->text().startsWith(","))
            brokerLineEdit->setText(brokerLineEdit->text().remove(0,1));
        else if (brokerLineEdit->text().endsWith(","))
            brokerLineEdit->setText(brokerLineEdit->text().remove(brokerLineEdit->text().length()-1,1));
        else
            brokerLineEdit->setText(brokerLineEdit->text().replace(",,",","));
    	brokersList->takeItem(brokersList->currentRow());
	}
	else
	{
	    QMessageBox::critical(this, "Remove broker", "There are no brokers selected");
	}
	checkBrokerListState();
}

//----------------------------------------------------------------------------
void ConnectionPage::checkBrokerListState()
{
    bool state = (brokersList->count() != 0);
    this->removeBrokerButton->setEnabled(state);
    this->upBrokerButton->setEnabled(state);
    this->downBrokerButton->setEnabled(state);
}

//----------------------------------------------------------------------------
void ConnectionPage::editItem(const QModelIndex & index)
{
    bool ok;
    QString currentText = brokersList->currentItem()->text();
    QString text = QInputDialog::getText(this, "Broker address",
                                            "Broker address:                          ",
                                             QLineEdit::Normal, currentText, &ok);

    if (ok && !text.isEmpty() && text != currentText)
    {
    	text.remove("http://");
    	text.remove("https://");
        if (brokersList->findItems(text, Qt::MatchExactly).isEmpty())
    	{
            brokersList->currentItem()->setText(text);
            // correct the line edit text
            brokerLineEdit->setText(brokerLineEdit->text().replace(currentText,text));
        }
        else
        {
    	    QMessageBox::critical(this, "Brokers list", "The list already contains this address");
        }
    }
}

//----------------------------------------------------------------------------
void ConnectionPage::setBrokerList(const QString& brokers)
{
	if (brokersList->count() == 0)
	{
		QStringList list = brokers.split(",", QString::SkipEmptyParts);
    	for (int i = 0; i < list.size(); i++)
    	{
   			brokersList->addItem(list.at(i));
    	}
	}
	checkBrokerListState();
}

//----------------------------------------------------------------------------
void ConnectionPage::setSslEnabled(bool enabled)
{
    sslGatewayLineEdit->setEnabled(enabled);
    sslGatewayLabel->setEnabled(enabled);
    if (checkComplete() == 0)
    {
        parentDialog->setNextEnabled(true);
    }
    else
    {
        parentDialog->setNextEnabled(false);
    }
}
