
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "dialogpages.h"


DrivesMappingOptionsPage::DrivesMappingOptionsPage(ConnectionDialog *parent)
    : ConnectionDialogPage(parent)
{
    setTitle(tr("Client Drives"),
    		 tr("Specify local drives to be redirected to the remote host."));

    label = new QLabel(tr("Drives:"));
    list = new QListWidget;
    lineEdit = new QLineEdit;
    lineEdit->setVisible(false);
	MyPushButtonCreator creator;
    addButton = creator.createButton("Add", 120);
    removeButton = creator.createButton("Remove", 120);
    removeAllButton = creator.createButton("Remove all", 120);

    connect(addButton, SIGNAL(clicked()), this, SLOT(addItem()));
    connect(removeButton, SIGNAL(clicked()), this, SLOT(removeItem()));
    connect(removeAllButton, SIGNAL(clicked()), this, SLOT(removeAllItem()));
    connect(list, SIGNAL(doubleClicked ( const QModelIndex &)),
            this, SLOT(editItem(const QModelIndex &)));
    connect(lineEdit, SIGNAL(textChanged(const QString&)),
    		this, SLOT(populateList(const QString&)));

	registerField("drives", lineEdit);
    
    QVBoxLayout *vLayout = new QVBoxLayout;
    vLayout->addWidget(addButton);
    vLayout->addWidget(removeButton);
    vLayout->addWidget(removeAllButton);
    vLayout->setAlignment(Qt::AlignTop);

    QHBoxLayout *hLayout = new QHBoxLayout;
    hLayout->addWidget(list);
    hLayout->addLayout(vLayout);
    
    QVBoxLayout *layout = new QVBoxLayout;
    layout->addLayout(titleLayout);
    layout->addWidget(label);
    layout->addLayout(hLayout);
//    layout->addWidget(lineEdit);
    layout->setAlignment(Qt::AlignTop);
    setLayout(layout);

    checkList();
}

//----------------------------------------------------------------------------

void DrivesMappingOptionsPage::checkList()
{
    if (list->count() == 0)
    {
        removeAllButton->setEnabled(false);
        removeButton->setEnabled(false);
    }
    else
    {
        removeAllButton->setEnabled(true);
        removeButton->setEnabled(true);
    }
}

void DrivesMappingOptionsPage::addItem()
{
    const QString title(tr("Client drives list"));
    const QString incorrectPathMsg(tr("The typed drive path is incorrect.\n"
                                      "Please specify correct path or click Cancel."));

    bool ok;
    QString text;
    
    do
    {
        text = QInputDialog::getText(this, "Add Drive",
                                            "Drive to be redirected: (example: mydisk=/home/user)",
                                             QLineEdit::Normal, text, &ok);
        if (ok)
        {
            if (text.contains("="))
            {
                QStringList listOfValues = text.split("=");
                QString path(listOfValues.at(1));
                QDir dir(path);
                if (path.isEmpty() || !dir.exists())
                {
                    QMessageBox::critical(this, title, incorrectPathMsg);
                }
                else
                {
                text.remove("\"");
       	        break;
       	    }
       	    }
       	    else
       	    {
           	    QMessageBox::critical(this, title, tr("You should specify share name!"));
           	}
        }
    }
    while (ok);
    if (ok && !text.isEmpty()) {

        int i = 0;

        QStringList nameAndVal = text.split("=");
        QString name = nameAndVal.at(0);
        QString newName = nameAndVal.at(0);
        bool exists = false;

        if (!list->findItems(text, Qt::MatchExactly).isEmpty())
        {
            exists = true;
        }

        while(!list->findItems(newName + "=", Qt::MatchContains).isEmpty() && list->count() > 0)
        {
            ++i;
            newName = name + "(" + QString::number(i, 10) + ")";
            text = newName + "=" + nameAndVal.at(1);
            if (!list->findItems(text, Qt::MatchExactly).isEmpty())
            {
                exists = true;
                break;
            }
        }

        if (!exists)
	    {
    	    list->addItem(text);
    	    // embrace the text in double-quotes bcos it can contain whitespaces 
            if (lineEdit->text() == "")
	        {
    	        lineEdit->setText("\"" + text + "\"");
            }
	        else
    	    {
                lineEdit->setText(lineEdit->text() + ",\"" + text + "\"");
	        }
        }
    }
    checkList();
}

//----------------------------------------------------------------------------
void DrivesMappingOptionsPage::removeItem()
{
    if (list->currentItem() != NULL)
    {
        lineEdit->setText(lineEdit->text().remove(list->currentItem()->text()));
        if (lineEdit->text().startsWith("\"\","))
            lineEdit->setText(lineEdit->text().remove(0,3));
        else if (lineEdit->text().endsWith(",\"\""))
            lineEdit->setText(lineEdit->text().remove(lineEdit->text().length()-4,3));
        else
            lineEdit->setText(lineEdit->text().replace(",\"\",",","));
        lineEdit->setText(lineEdit->text().remove("\"\""));
    	list->takeItem(list->currentRow());
	}
	else
	{
	    QMessageBox::critical(this, "Remove item", "There are no items selected");
	}
	checkList();
}

//----------------------------------------------------------------------------
void DrivesMappingOptionsPage::removeAllItem()
{
    int ret = QMessageBox::warning(this, tr("Client drives"),
                   tr("You're going to remove all items in the list.\nRemove?"),
                   QMessageBox::Yes | QMessageBox::No);
    if (ret == QMessageBox::Yes)
    {
        lineEdit->setText("");
        list->clear();
    }
    checkList();
}

//----------------------------------------------------------------------------
void DrivesMappingOptionsPage::editItem(const QModelIndex & index)
{
    const QString title(tr("Client drives list"));
    const QString incorrectPathMsg(tr("The typed drive path is incorrect.\n"
                                      "Please specify correct path or click Cancel."));


    bool ok;
    QString currentText = list->currentItem()->text();
    QString text = currentText; 
    
    do
    {
        text = QInputDialog::getText(this, "Add Drive",
                                            "Drive to be redirected: (example: mydisk=/home/user)",
                                             QLineEdit::Normal, text, &ok);
        if (ok)
        {
            if (text.contains("="))
            {
                QStringList listOfValues = text.split("=");
                QString path(listOfValues.at(1));
                QDir dir(path);
                if (path.isEmpty() || !dir.exists())
                {
                    QMessageBox::critical(this, title, incorrectPathMsg);
                }
                else
                {
                text.remove("\"");
       	        break;
       	    }
       	    }
       	    else
       	    {
           	    QMessageBox::critical(this, title, "You should specify share name!");
           	}
        }
    }
    while (ok);
    if (ok && !text.isEmpty())
    {
        if (text != currentText)
        {
            if (list->findItems(text, Qt::MatchExactly).isEmpty())
        	{
                list->currentItem()->setText(text);
                // correct the line edit text
                lineEdit->setText(lineEdit->text().replace(currentText,text));
            }
            else
            {
        	    QMessageBox::critical(this, "Client drives list", "The list already contains this item");
            }
        }
    }
    checkList();
}

//----------------------------------------------------------------------------
void DrivesMappingOptionsPage::populateList(const QString& items)
{
    QString string = items; 	
	if (list->count() == 0)
	{
		QStringList templist = string.remove("\"").split(",", QString::SkipEmptyParts);
    	for (int i = 0; i < templist.size(); i++)
    	{
   			list->addItem(templist.at(i));
    	}
	}
	checkList();
}
