
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "dialogpages.h"


SessionNamePage::SessionNamePage(ConnectionDialog *parent)
    : ConnectionDialogPage(parent)
{
	QFont font;
	font.setBold(true);

	setTitle(tr("Connection Name"), tr("Specify a name for the connection."));

    sessionNameLabel = new QLabel(tr("Connection name:"));
    sessionNameLineEdit = new QLineEdit;
    sessionNameLineEdit->setMaxLength(100);

    connect (sessionNameLineEdit, SIGNAL(textChanged(const QString &)),
             this, SLOT(sessionNameChanged(const QString &)));
    connect (sessionNameLineEdit, SIGNAL(editingFinished()),
             this, SLOT(checkName()));

    registerField("sessionname*", sessionNameLineEdit);

    QHBoxLayout *layout = new QHBoxLayout;
    layout->addWidget(sessionNameLabel);
    layout->addWidget(sessionNameLineEdit);
    QVBoxLayout *vlayout = new QVBoxLayout;
    vlayout->addLayout(titleLayout);
    vlayout->addLayout(layout);
    vlayout->setAlignment(Qt::AlignTop);
    setLayout(vlayout);
}

//----------------------------------------------------------------------------
void SessionNamePage::checkName()
{
    if (sessionNameLineEdit->text() == currentSessionName)
    {
        return;
    }

    QStringList files;
    QString fileName = "*.rdp";
    QDir dir(QDir::current());
    files = dir.entryList(QStringList(fileName), QDir::Files | QDir::NoSymLinks);
    int i = 0;
    for (; i < files.size(); i++)
    {
   		files[i].truncate(files[i].length() - 4);
   		if (files[i] == sessionNameLineEdit->text())
   		{
            QMessageBox::critical(this, "Wrong name", "This name is already used by another connection.");
           	sessionNameLineEdit->clear();
           	sessionNameLineEdit->setFocus();
           	return;
   		}
    }

    if (sessionNameLineEdit->text().trimmed().isEmpty())
    {
        return;
    }

    ConnectionEditor *editor = NULL;
    if ((editor = qobject_cast <ConnectionEditor *> (parentDialog)) != NULL)
    {
        QFile *file = new QFile(QDir::current().absolutePath() + QDir::separator() + currentSessionName + ".rdp");
        file->remove();
    }
}

//----------------------------------------------------------------------------
void SessionNamePage::sessionNameChanged(const QString &text)
{
    if (currentSessionName.isEmpty())
    {
        currentSessionName = sessionNameLineEdit->text();
    }
}
