
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "dialogpages.h"


SmartCardsOptionsPage::SmartCardsOptionsPage(ConnectionDialog *parent)
    : ConnectionDialogPage(parent)
{
    setTitle(tr("Smart Cards"),
    		 tr("Specify smart cards to be redirected to the remote host."));

    checkBox = new QCheckBox("Enable smart cards mapping");
    checkBox->setChecked(true);
    label = new QLabel(tr("List of smart card names (optional):"));

    list = new QListWidget;
    lineEdit = new QLineEdit;
    lineEdit->setVisible(false);
	MyPushButtonCreator creator;
    addButton = creator.createButton("Add", 120);
    removeButton = creator.createButton("Remove", 120);
    removeAllButton = creator.createButton("Remove all", 120);

    connect(checkBox, SIGNAL(stateChanged(int)),
            this, SLOT(setListEnabled(int)));
    connect(addButton, SIGNAL(clicked()), this, SLOT(addItem()));
    connect(removeButton, SIGNAL(clicked()), this, SLOT(removeItem()));
    connect(removeAllButton, SIGNAL(clicked()), this, SLOT(removeAllItem()));
    connect(list, SIGNAL(doubleClicked ( const QModelIndex &)),
            this, SLOT(editItem(const QModelIndex &)));
    connect(lineEdit, SIGNAL(textChanged(const QString&)),
    		this, SLOT(populateList(const QString&)));

	registerField("scardEnabled", checkBox);
	registerField("smartcards", lineEdit);

    QVBoxLayout *vLayout = new QVBoxLayout;
    vLayout->addWidget(addButton);
    vLayout->addWidget(removeButton);
    vLayout->addWidget(removeAllButton);
    vLayout->setAlignment(Qt::AlignTop);

    QHBoxLayout *hLayout = new QHBoxLayout;
    hLayout->addWidget(list);
    hLayout->addLayout(vLayout);

    QVBoxLayout *layout = new QVBoxLayout;
    layout->addLayout(titleLayout);
    layout->addWidget(checkBox);
    layout->addWidget(label);
    layout->addLayout(hLayout);
//    layout->addWidget(lineEdit);
    layout->setAlignment(Qt::AlignTop);
    setLayout(layout);
}

//----------------------------------------------------------------------------
void SmartCardsOptionsPage::addItem()
{
    bool ok;
    QString text;

    text = QInputDialog::getText(this, "Add Smart Card Name",
                                        "Smart card to be redirected: \n\
    (in the form of \"Scard Name\"=\"Alias Name[;Vendor Name]\"[,...])\n\n\
Examples:\n\
    \"eToken PRO 00 00\"=\"AKS ifdh 0\"\n\
    \"eToken PRO 00 00\"=\"AKS ifdh 0;AKS\"",
        QLineEdit::Normal, text, &ok);
    if (ok && !text.isEmpty()) {
        embraceWithQuotes(text);
        if (list->findItems(text, Qt::MatchExactly).isEmpty())
	    {
    	    list->addItem(text);
            if (lineEdit->text() == "")
	        {
    	        lineEdit->setText(text);
            }
	        else
    	    {
                lineEdit->setText(lineEdit->text() + "," + text);
	        }
        }
        else
        {
            QMessageBox::critical(this, "Smart cards list", "The list already contains this item");
        }
    }
}

//----------------------------------------------------------------------------
void SmartCardsOptionsPage::removeItem()
{
    if (list->currentItem() != NULL)
    {
        lineEdit->setText(lineEdit->text().remove(list->currentItem()->text()));
        if (lineEdit->text().startsWith(','))
            lineEdit->setText(lineEdit->text().remove(0,1));
        else if (lineEdit->text().endsWith(','))
            lineEdit->setText(lineEdit->text().remove(lineEdit->text().length()-1,1));
        else
            lineEdit->setText(lineEdit->text().replace(",,",","));
    	list->takeItem(list->currentRow());
	}
	else
	{
	    QMessageBox::critical(this, "Remove item", "There are no items selected");
	}
}

//----------------------------------------------------------------------------
void SmartCardsOptionsPage::removeAllItem()
{
    int ret = QMessageBox::warning(this, tr("Smart Cards"),
                   tr("You're going to remove all items in the list.\nRemove?"),
                   QMessageBox::Yes | QMessageBox::No);
    if (ret == QMessageBox::Yes)
    {
        lineEdit->setText("");
        list->clear();
    }
}

//----------------------------------------------------------------------------
void SmartCardsOptionsPage::editItem(const QModelIndex & index)
{
    bool ok;
    QString currentText = list->currentItem()->text();
    QString text = currentText;

    text = QInputDialog::getText(this, "Add Smart Card",
                                        "Smart card to be redirected: \n\
    (in the form of \"Scard Name\"=\"Alias Name[;Vendor Name]\"[,...])\n\n\
Examples:\n\
    \"eToken PRO 00 00\"=\"AKS ifdh 0\"\n\
    \"eToken PRO 00 00\"=\"AKS ifdh 0;AKS\"",
        QLineEdit::Normal, text, &ok);
    if (ok && !text.isEmpty())
    {
        embraceWithQuotes(text);
        if (text != currentText)
        {
            if (list->findItems(text, Qt::MatchExactly).isEmpty())
        	{
                list->currentItem()->setText(text);
                // correct the line edit text
                lineEdit->setText(lineEdit->text().replace(currentText,text));
            }
            else
            {
        	    QMessageBox::critical(this, "Smart cards list", "The list already contains this item");
            }
        }
    }
}

//----------------------------------------------------------------------------
void SmartCardsOptionsPage::populateList(const QString& items)
{
    QString string = items;
	if (list->count() == 0)
	{
		QStringList templist = string.split(',', QString::SkipEmptyParts);
    	for (int i = 0; i < templist.size(); i++)
    	{
   			list->addItem(templist.at(i));
    	}
	}
}

//----------------------------------------------------------------------------
void SmartCardsOptionsPage::setListEnabled(const int state)
{
    if (state == Qt::Checked)
    {
        list->setEnabled(true);
        label->setEnabled(true);
        lineEdit->setEnabled(true);
        addButton->setEnabled(true);
        removeButton->setEnabled(true);
        removeAllButton->setEnabled(true);
    }
    else
    {
        list->setEnabled(false);
        label->setEnabled(false);
        lineEdit->setEnabled(false);
        addButton->setEnabled(false);
        removeButton->setEnabled(false);
        removeAllButton->setEnabled(false);
    }
}

//----------------------------------------------------------------------------
void SmartCardsOptionsPage::embraceWithQuotes(QString& text)
{
    text.remove('"');

    QString strA = text.left(text.indexOf('='));
    QString strB = text.right(text.length() - text.indexOf('=') - 1);

    if (strA.contains(' '))
    {
        text.insert(0, '"');
        text.insert(text.indexOf('='), '"');
    }
    if (strB.contains(' '))
    {
        text.insert(text.indexOf('=')+1, '"');
        text = text + '"';
    }
}
