#!/bin/sh

# some variables

# general
SUCCESS=0
NO_SUCCESS=255
enable_mmr=0
enable_fmmr=0
enable_mic=0
enable_usb=0
enable_mm=0
enable_proxying=0
build_gui=0
pkg_util=
test_pkg=
find_pkg=
install_pkg=
gst_inspect=
current_dist=
kernel_sources_dir=
kernel_sources_dir_prefix=
absent_packages=

# libs
GST_MAIN_MODULE=gstreamer-0.10
GST_VERSION=
GENERAL_LIBS=
GST_MODULES=
GST_DEV_LIBS=
USB_LIBS=
MIC_LIBS=
FMMR_LIBS=
OTHER_LIBS=
PROXYING_LIBS=
MM_LIBS=
QT_CORE=QtCore
QT_VERSION=4.3.3
QT_LIBS=
QT_ALT_LIBS=
QT_QMAKE=

# functions

test_pkg_debian()
{
  dpkg-query -W -f='${Status}' $@ 2>/dev/null | grep " installed" 1>/dev/null
  if [ $? != $SUCCESS ]
  then
    # try another way - using pkg-config
    PKG=$(echo "$@" | sed 's/-dev//')
    pkg-config --exists $PKG
  fi
  return $?
}

test_pkg_redhat()
{
  $pkg_util $@ 1>/dev/null 2>/dev/null
  if [ $? != $SUCCESS ]
  then
    # try another way - using pkg-config
    PKG=$(echo "$@" | sed 's/-devel//')
    pkg-config --exists $PKG
  fi
  return $?
}

install_pkg_redhat()
{
  sudo yum -y install $@
  return $?
}

install_pkg_debian()
{
  sudo apt-get install `aptitude search -F "%p" $@`
  return $?
}

find_pkg_redhat()
{
  yum list available $@ 1>/dev/null 2>/dev/null
  return $?
}

find_pkg_debian()
{
  if [ "`aptitude search $@ 2>/dev/null`" != "" ]
  then
    return $SUCCESS
  else
    return $NO_SUCCESS
  fi
}

# test given list of package names checking if all of them are installed
test_pkg_list()
{
  absent_packages=""
  for pkg_name in $@
  do
    echo -n " checking for $pkg_name... "
    $test_pkg $pkg_name
    if [ $? -ne $SUCCESS ]
    then
      echo -n " not installed... try to install? (y/n): "
      read response
      if [ \( "$response" = "Y" \) -o \( "$response" = "y" \) ]
      then
        $find_pkg $pkg_name
        if [ $? -eq $SUCCESS ]
        then
          $install_pkg $pkg_name
          if [ $? -ne $SUCCESS ]
          then
            echo "  Installation failed - you can try to install it manually"
          else
            echo "  succesfully installed"
            echo
            continue
          fi
        else
          echo "  ERROR: package $pkg_name is not found, try to install it manually"
        fi
      fi
      absent_packages="$absent_packages ${pkg_name}"
    else
      echo -e "\tOK"
    fi
  done
  if [ "$absent_packages" != "" ]
  then
    echo
    echo "  Absent packages: $absent_packages"
    echo
  fi
}

# main code

echo
echo '========================================='
echo 'Beginning qrdesktop prerequisites checker'
echo '========================================='
echo 

# determine the current distribution and set variables
if [ -f /etc/redhat-release ]
then
  # set variables for RH
  GST_VERSION=0.10.19
  pkg_util="rpm -q"
  test_pkg=test_pkg_redhat
  find_pkg=find_pkg_redhat
  install_pkg=install_pkg_redhat
  GENERAL_LIBS="gcc-c++ libstdc++ libpng popt openssl boost libX11 dos2unix autoconf libao alsa-lib libmpg123 xdg-utils wget"
  GENERAL_DEV_LIBS="libstdc++-devel libpng-devel openssl-devel boost-devel libX11-devel libao-devel alsa-lib-devel libmpg123-devel"
  if grep 'Fedora' /etc/redhat-release > /dev/null
  then
    GENERAL_DEV_LIBS="$GENERAL_DEV_LIBS popt-devel asio-devel"
  fi
  GST_MODULES="gstreamer-0.10* gstreamer-plugins-base-0.10* gstreamer-tools-0.10* gstreamer-plugins-good-0.10*"
  GST_DEV_LIBS="gstreamer-devel-0.10* gstreamer-plugins-base-devel-0.10* libXext-devel"
  gst_inspect=gst-inspect
  USB_LIBS="kernel-devel-`uname -r`"
  MIC_LIBS="speex-devel"
  FMMR_LIBS="gtk2-devel flash-plugin libXt-devel"
  QT_LIBS="qt-4* qt-x11-4* qt-devel-4*"
  QT_ALT_LIBS="qt4 qt4-x11 qt4-devel"
  QT_QMAKE=qmake-qt4
  MM_LIBS="libXinerama-devel"
  current_dist="redhat"
  kernel_sources_dir="/usr/src/kernels"

  KERNEL_PATH_1="$(uname -r)"
  KERNEL_PATH_2="$(uname -r)-$(uname -p)"
  KERNEL_PATH_3="$(uname -r).$(uname -p)"
  if [ -d $kernel_sources_dir/$KERNEL_PATH_1 ]
  then
    kernel_sources_dir_prefix=$KERNEL_PATH_1
  elif [ -d $kernel_sources_dir/$KERNEL_PATH_2 ]
  then
    kernel_sources_dir_prefix=$KERNEL_PATH_2
  elif [ -d $kernel_sources_dir/$KERNEL_PATH_3 ]
  then
    kernel_sources_dir_prefix=$KERNEL_PATH_3
  fi

elif [ -f /etc/SuSE-release ]
then
  # set variables for SuSE
  GST_VERSION=0.10.19
  pkg_util="rpm -q"
  GENERAL_LIBS="gcc-c++ stdc++-devel png-devel popt-devel openssl-devel boost-devel jpeg-devel xorg-x11-devel xdg-utils" # xorg-x11-Xext-devel libasio-dev"
  if grep 'openSUSE' /etc/SuSE-release > /dev/null
  then 
    # openSUSE
    GST_MODULES="gstreamer-0_10 gstreamer-0_10-plugins-base gstreamer-0_10-utils gstreamer-0_10-plugins-good"
    GST_DEV_LIBS="gstreamer-0_10-devel gstreamer-0_10-plugins-base-devel"
    gst_inspect=gst-inspect-0.10
  else
    # SLED or SLETC
    GST_MODULES="gstreamer010 gstreamer010-plugins-base gstreamer010-utils gstreamer010-plugins-good"
    GST_DEV_LIBS="gstreamer010-devel gstreamer010-plugins-base-devel"
    gst_inspect=gst-inspect
  fi
  USB_LIBS="kernel-source"
  MIC_LIBS="speex-devel"
  FMMR_LIBS="gtk2-devel flash-plugin"
  QT_LIBS="libqt4 libqt4-x11 libqt4-devel"
  QT_QMAKE=qmake
  current_dist="redhat" # not an error!
  kernel_sources_dir="/usr/src/linux"
elif [ -f /etc/debian_version ]
then
  # set variables for Debian and Ubuntu
  GST_VERSION=0.10.18
  pkg_util="dpkg-query -l"
  test_pkg=test_pkg_debian
  find_pkg=find_pkg_debian
  install_pkg=install_pkg_debian
  GENERAL_LIBS="g++ libstdc++6* libpng12 libpopt* openssl libboost-thread1.34* libboost-regex* libboost-date-time* libx11-* tofrodos autoconf libjpeg62 libao* libasound2 libmpg123* xdg-utils"
  GENERAL_DEV_LIBS="libstdc++6*-dev libpng12-dev libpopt*-dev openssl libboost-dev libboost-thread-dev libboost-regex-dev libboost-date-time-dev libx11-dev libasio-dev tofrodos autoconf libjpeg62-dev libao-dev libasound2-dev libmpg123-dev"
  GST_MODULES="libgstreamer0.10-0 gstreamer0.10-x gstreamer0.10-plugins-base gstreamer0.10-tools gstreamer0.10-plugins-good"
  GST_DEV_LIBS="libgstreamer0.10-dev libgstreamer-plugins-base0.10-dev libxext-dev"
  gst_inspect=gst-inspect-0.10
  USB_LIBS="linux-headers-`uname -r`"
  MIC_LIBS="libspeex-dev"
  FMMR_LIBS="libgtk2.0-dev flashplugin-nonfree* libxt-dev"
  QT_LIBS="libqt4-dev libqt*gui* libqt*core*"
  QT_QMAKE=qmake-qt4
  MM_LIBS="libxinerama-dev"
  current_dist="debian"
  kernel_sources_dir="/usr/src"
  kernel_sources_dir_prefix="linux-headers-`uname -r`"
else
  # we don't support other distros
  echo 'Error: unknown Linux distribution.'
  exit 1
fi

sleep 1

# Sanity checks begin here
echo '=================================='
echo 'Performing Pre-requisite checks...'
echo '=================================='
sleep 1

if ! [ -e /usr/bin/pkg-config ]
then
  echo
  echo 'Cannot find pkg-config - please install pkg-config before continue installation'
  exit
fi

PKG_CONFIG_PATH=$PKG_CONFIG_PATH:/usr/local/lib/pkgconfig:/usr/lib/pkgconfig
export PKG_CONFIG_PATH

echo ''
echo 'Checking for core libraries:'
test_pkg_list $GENERAL_LIBS
echo ''
echo ''

if [ "$#" -eq "1" ]
then
  if [ "$1" != "BINARY" ]
  then
    echo 'Checking core development libraries'
    test_pkg_list $GENERAL_DEV_LIBS
    echo ''
    echo 'Done checking development packages'
  fi
else
  echo 'Checking core development libraries'
  test_pkg_list $GENERAL_DEV_LIBS
  echo ''
  echo 'Done checking development packages'
fi

if [ "$absent_packages" != "" ]
then
  echo 
  echo "Some of the core libraries that are required are not present."
  echo
  if [ "$1" != "-force-build" ]
  then
    exit
  fi
else
  echo ' OK'
  echo "All core libraries available. Core components can be installed"
  echo
fi
sleep 1

if [ ! -f /usr/lib/libmpg123.so.0 ]
then
  sudo ln -s /usr/local/lib/libmpg123.so.0 /usr/lib/libmpg123.so.0
fi

# check for GStreamer
echo 'checking for GStreamer...'
test_pkg_list $GST_MODULES $GST_DEV_LIBS
if [ "$absent_packages" = "" ]
then
  if pkg-config --atleast-version=$GST_VERSION $GST_MAIN_MODULE
  then
    enable_mmr=1
  else
    echo 'warning: GStreamer-0.10.19 or later is required'
  fi
fi
if [ $enable_mmr -ne 1 ]
then
  echo
  echo 'Warning: either GStreamer of some of its modules is not installed or does not match version requirements. MMR extension will not work.'
  echo
else
  echo ' OK'
fi

echo 'checking for GStreamer plugins...'

#check for gstreamer ffmpeg library
if ! $gst_inspect ffmpeg 1>/dev/null 2>/dev/null
then
  echo ' Warning: GStreamer FFMpeg library is absent - some of media formats may not play'
fi

#check for gstreamer pitfdll library (windows codecs also need checking...)
if ! $gst_inspect pitfdll 1>/dev/null 2>/dev/null
then
  echo ' Warning: GStreamer pitfdll library is absent - WMA9 Pro, Voice and Loseless streams will not play'
fi
if ! [ -e /usr/lib/codecs -o -e /usr/lib/win32 ]
then
  echo ' Warning: GStreamer Win32 codecs is absent - WMA9 Pro, Voice and Loseless streams will not play'
fi
echo ' done'

#check for Flash MMR stuff
echo
echo 'checking for Flash MMR development libs...'
test_pkg_list $FMMR_LIBS
if [ "$absent_packages" = "" ]
then
  enable_fmmr=1
  echo ' OK'
else
  echo
  echo ' Warning: one or more libraries needed for Flash MMR support is not installed. Flash MMR extension will not be installed.'
  echo
fi
sleep 1

#check for microphone stuff
echo 'checking for audio development libs...'
test_pkg_list $MIC_LIBS
if [ "$absent_packages" = "" ]
then
  enable_mic=1
  echo ' OK'
else
  echo
  echo ' Warning: one or more libraries needed for microphone redirection is not installed. Microphone redirection extension will not work.'
  echo
fi

#check for USB stuff
echo 'checking for USB development libs...'
test_pkg_list $USB_LIBS
if [ "$absent_packages" = "" ]
then
  if [ -d $kernel_sources_dir/$kernel_sources_dir_prefix ]
  then
    enable_usb=1
    echo ' OK'
  else
    echo
    echo " Warning: development files for kernel $(uname -r) not found in $kernel_sources_dir. USB redirection extension will not work."
    echo
  fi
else
  echo
  echo ' Warning: one or more libraries needed for USB redirection is not installed. USB redirection extension will not work.'
  echo
fi

# check for QT libs
echo 'checking for QT libraries'

# don't waste time on searching QT in the package database on RHEL
if ! grep 'Red Hat' /etc/redhat-release > /dev/null
then
    test_pkg_list $QT_LIBS
    if [ "$absent_packages" = "" ]
    then
      build_gui=1
    else
  # check other QT libs
	test_pkg_list $QT_ALT_LIBS
	if [ "$absent_packages" = "" ]
	then
    	build_gui=1
    	fi
    fi
fi

if [ $build_gui -ne 1 ]
then
# try another way to find QT (it might have been installed from sources)
  echo -n " searching for QT libraries in pkgconfig directory..."
  if pkg-config --atleast-version=4.4 QtGui
  then
    echo -e "\tOK"
    build_gui=1
  else
    echo -e "\tnot found"
  fi
fi

if [ $build_gui -ne 1 ] || ! pkg-config --atleast-version=$QT_VERSION $QT_CORE
then
  echo
  echo ' Warning: QT is not installed or does not match version requirements. Connection manager GUI will not be built.'
  echo
else
  echo ' OK'
fi

echo
echo '========================='
echo 'Prerequisites checks done'
echo '========================='
echo

