
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#ifndef _QAUDIO_DRIVER_H_
#define _QAUDIO_DRIVER_H_

#define QDRIVER_ALSA  1
#define QDRIVER_OSS   2
#define QDRIVER_LIBAO 3
#define QDRIVER_SGI   4
#define QDRIVER_SUN   5

typedef struct
{
    uint32 low;
    uint32 high;
}uint64;

#define in_uint64_le(s , v ) in_uint32_le((s), (v).low); in_uint32_le((s), (v).high );


typedef struct tWAVEFORMATEX
{
    uint16 wFormatTag;
    uint16 nChannels;
    uint32 nSamplesPerSec;
    uint32 nAvgBytesPerSec;
    uint16 nBlockAlign;
    uint16 wBitsPerSample;
    uint16 cbSize;    
} WAVEFORMATEX, * LPWAVEFORMATEX;

typedef void* HWAVEOUT;
typedef HWAVEOUT* LPHWAVEOUT;

typedef void * HWAVEIN;
typedef HWAVEIN* LPHWAVEIN;

typedef int ( * pwaveOutOpen  ) ( LPHWAVEOUT _handle, WAVEFORMATEX * _format );
typedef int ( * pwaveOutReset ) ( HWAVEOUT _handle );
typedef int ( * pwaveOutClose ) ( HWAVEOUT _handle );
typedef int ( * pwaveOutWrite ) ( HWAVEOUT _handle , uint8 * _wave , uint32 _size );

typedef int ( * pwaveInOpen   ) ( LPHWAVEIN _handle , WAVEFORMATEX * _format );
typedef int ( * pwaveInClose  ) ( HWAVEIN _handle );
typedef int ( * pwaveInStart  ) ( HWAVEIN _handle );
typedef int ( * pwaveInStop   ) ( HWAVEIN _handle );
typedef int ( * pwaveInRecord ) ( HWAVEIN _handle , uint8 * _data , uint32 _size  );

typedef struct
{
    int driver;
    
    pwaveOutOpen  waveOutOpen;
    pwaveOutClose waveOutClose;
    pwaveOutReset waveOutReset;
    pwaveOutWrite waveOutWrite;
    
    pwaveInOpen   waveInOpen;
    pwaveInClose  waveInClose;
    pwaveInStart  waveInStart;
    pwaveInStop   waveInStop;
    pwaveInRecord waveInRecord;

} qaudio_waveout_driver, qaudio_wavein_driver;

extern qaudio_waveout_driver waveout_driver;
extern qaudio_wavein_driver wavein_driver;

int qaudio_driver_initialize ( int _driver, char * _options );
int qaudio_driver_uninitialize ();

#endif
