
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


#include "rdesktop.h"
#include "qaudio_libao_drv.h"
#include <ao/ao.h>

typedef struct
{
    ao_device * handle;
} libao_waveout_handle;


static int device_id = 0;

 
int libao_driver_init ( char * _options )
{
	ao_initialize();

	if ( _options == NULL )
	{
		device_id = ao_default_driver_id();
	}
	else
	{
		device_id = ao_driver_id (_options);
	}

	return 0;
}
int libao_driver_free ()
{
	ao_shutdown();
    return 0;
}

int libao_waveout_open  ( LPHWAVEOUT _handle, WAVEFORMATEX * _format )
{
	ao_sample_format format;
	libao_waveout_handle * handle = (libao_waveout_handle*)xmalloc(sizeof( libao_waveout_handle));

	format.bits 	   = _format->wBitsPerSample;
	format.channels    = _format->nChannels;
	format.rate 	   = _format->nSamplesPerSec;
	format.byte_format = AO_FMT_LITTLE;

	handle->handle = ao_open_live( device_id , &format , NULL );

	if ( handle->handle == NULL)
	{
		xfree ( handle );
		return -1;
	}

	*_handle = (HWAVEOUT)handle;

    return 0;
}

int libao_waveout_reset ( HWAVEOUT _handle )
{
    return 0;
}

int libao_waveout_close ( HWAVEOUT _handle )
{
	libao_waveout_handle * handle = (libao_waveout_handle*) _handle;
	if ( handle == NULL )
		return -1;

	ao_close( handle->handle );

	xfree ( handle );

    return 0;
}

int libao_waveout_write ( HWAVEOUT _handle , uint8 * _wave , uint32 _size )
{
	int ret = 0;
	libao_waveout_handle * handle = (libao_waveout_handle*) _handle;
	if ( handle == NULL )
		return 0;

	ret = ao_play( handle->handle ,(char*) _wave, (uint_32)_size );
	if ( ret < 0 )
		return 0;

    return ret;
}
