
/******************************************************************************
 *
 *  Quest Software, Inc.
 * ALL RIGHTS RESERVED.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *  *  Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.  
 * 
 *  *  Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.  
 * 
 *  *  Neither the name of Quest Software, Inc. nor the names of its contributors 
 *     may be used to endorse or promote products derived from this software 
 *     without specific prior written permission. 
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
 * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
 * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 ************************************************************************************/


/*
   Copyright (c) 2007 Provision Networks.
   http://www.provisionnetworks.com

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License;
   either version 2 of the License, or any later version.
 */

#include <stdio.h>
#include <time.h>
#include <syslog.h>
#include <errno.h>
#include <string.h>
#include "../common/simplexml.h"
#include "../common/crypto.h"
#include "../common/errors.h"
#include "tmp.h"

#include "pnllmcli.h"

#define MAX_RESPONSE_SIZE   ( 2*1024*1024 )

static void ConvertWinErrcode2Text(int dwErrorCode, SimpleString &strErrorText)
{
    switch(dwErrorCode)
    {
    // Provision error codes
    case 42001:
        strErrorText = "The computer is powering up...";
        break;
    case 42002:
        strErrorText = "The computer operating system is loading...";
        break;
    case 42003:
        strErrorText = "The computer is not responding.";
        break;
    case 42004:
        strErrorText = "There are no available computers to connect to at this time.  Please report this message to your system administrator.";
        break;
    case 42005:
        strErrorText = "Your request is being processed...";
        break;
    case 42008:
        strErrorText = "The computer remote display protocol port is not reachable.";
        break;
    case 42009:
        strErrorText = "The computer or computer group has been disabled.";
        break;
    // Windows error codes
    case 1244:
        strErrorText = "The operation being requested was not performed because the user has not been authenticated.";
        break;
    case 1064:
        strErrorText = "An exception occurred in the service when handling the control request.";
        break;
    case 4313:
        strErrorText = "Unable to read from or write to the database.";
        break;
    case 5:
        strErrorText = "Access is denied.";
        break;
    case 5005:
        strErrorText = "A cluster node is not available for this operation.";
        break;
    case 1239:
        strErrorText = "Attempting to log in during an unauthorized time of day for this account.";
        break;
    case 1234:
        strErrorText = "No service is operating at the destination network endpoint on the remote system.";
        break;
    default:
        strErrorText = "Unknown error: ";
        strErrorText.Append("%s", dwErrorCode);
    }
}

int SendRequest(char* server, char *proxy, char* request, SimpleString& strResponse)
{
    char url[256];
    strcpy(url, server);
    if (strchr(url + 8, '/') == 0)
    {
        strcat(url, "/");
    }
    char* buffer = (char*)malloc(MAX_RESPONSE_SIZE);
    int r;
    int outputbufsize = MAX_RESPONSE_SIZE;
    if (strncmp(url, "https://", 8) == 0)
    {
        r = sweb(url, request, strlen(request), buffer, &outputbufsize, 0);
    }
    else
    {
        r = web(url, proxy, request, strlen(request), buffer, &outputbufsize, 0);
    }
    if (outputbufsize > 0)
    {
        strResponse.LoadData(buffer, outputbufsize);
    }
    else
    {
        strResponse.LoadData(buffer);
    }
    free(buffer);
    return r;
}
/*
static unsigned int Checksum(unsigned char* pBuffer, int cbBuffer)
{
    unsigned int dwChecksum = 0;

    for (int i = 0; i < cbBuffer; i++)
    {
        dwChecksum += pBuffer[i];
    }

    return dwChecksum % 256;
}
*/
int Authenticate(char* server, char *proxy, char* username, char* password, char* domain, char *tgt, bool nat, SimpleString& cookie)
{
    int ret = 0;
    SimpleString buffer;
    SimpleString strResponse;

    const char* szClientVersion = "7.0";
    const char* szClientIPAddress = "";
    const char* szClientName = "Linux";

    // Encrypt the password.
    unsigned char rgbBuffer[256];
    char szPassword[257 * 2];
    Encode256(rgbBuffer, password);
    Encrypt(rgbBuffer, sizeof(rgbBuffer));
    BinToHex(rgbBuffer, sizeof(rgbBuffer), szPassword, sizeof(szPassword));

    if (tgt)
    {
        buffer.Format(
            "<authenticate-req><tickets>%s</tickets>",
            tgt);
    }
    else
    {
        buffer.Format(
            "<authenticate-req><domain>%s</domain>"
                "<username>%s</username>",
            domain,
            username);

        buffer.Append(
    		"<password>%s</password>",
            szPassword);
    }

    buffer.Append(
			"<clientipaddress>%s</clientipaddress>"
            "<clientname>%s</clientname>"
            "<clientversion>%s</clientversion>"
            "<clientbitsperpixel>%d</clientbitsperpixel>"
            "<usealternateipaddress>%d</usealternateipaddress>",
        szClientIPAddress,
        szClientName,
        szClientVersion,
        16,
        nat ? 1 : 0);

    buffer.AppendData("</authenticate-req>");

    errno = 0;
    ret = SendRequest(server, proxy, buffer, strResponse);

#ifdef DEBUG
    syslog(LOG_NOTICE, "Request:%s", (char*)buffer);
    syslog(LOG_NOTICE, "Response:%s", (char*)strResponse);
#endif

    if (ret != 0)
    {
        cookie = strResponse;
        return ret;
    }

    // Parse the response.
    int dwErrorCode = 0;
    SimpleString strErrorText;
    SimpleXML xml, tag, data;

    xml.LoadData(strResponse);
    if (xml.FindTag("authenticate-rsp", tag, data) <= 0)
    {
        cookie = "Authentication information does not present in broker response";
        return BROKER_ERROR;
    }

    xml.Copy(data);

    if (xml.FindTag("errorcode", tag, data) > 0)
    {
        dwErrorCode = strtol(data.GetData(), 0, 10);
    }
    if (xml.FindTag("errortext", tag, data) > 0)
    {
        strErrorText = data.GetData();
    }
    else
    {
        ConvertWinErrcode2Text(dwErrorCode, strErrorText);
    }
    if (dwErrorCode)
    {
        cookie = (char*)strErrorText;
        return dwErrorCode;
    }

    if (xml.FindTag("cookie", tag, data) > 0)
    {
        cookie = data.GetData();
    }
    return 0;
}

int GetApplicationList(char* server, char *proxy, char* cookie, SimpleString& res, int type)
{
    int ret;
    _tmpfile tempFile("/tmp");

    //tempFile.CleanDir("AppPortal");

    //FILE* fSelfTempFile;
    SimpleString buffer;
    SimpleString strResponse;

    if (type == 1)
    {
        const int applsAndContentsType = 13;
        type = applsAndContentsType;
    }

    buffer.Format(
        "<getapplicationlist-req><cookie>%s</cookie><appfilter>%d</appfilter></getapplicationlist-req>",
        cookie,
        type ? type : 0xF);

    // Send the request.
    ret = SendRequest(server, proxy, buffer, strResponse);
    if (ret != 0)
    {
        res = strResponse;
        return ret;
    }

    // Parse the response.
    int dwErrorCode = 0;
    SimpleString strErrorText;
    SimpleXML xml, tag, data;
    int start = 0;

    xml.LoadData(strResponse);
    if (xml.FindTag("getapplicationlist-rsp", tag, data) <= 0)
    {
        return BROKER_ERROR;
    }

    xml.Copy(data);

    if (xml.FindTag("errorcode", tag, data) > 0)
    {
        dwErrorCode = strtol(data.GetData(), 0, 10);
    }
    if (xml.FindTag("errortext", tag, data) > 0)
    {
        strErrorText = data.GetData();
    }
    else
    {
        ConvertWinErrcode2Text(dwErrorCode, strErrorText);
    }
    if (dwErrorCode)
    {
        res = (char*)strErrorText;
        return dwErrorCode;
    }

    while ((start = xml.FindTag("application", tag, data, start)) > 0)
    {
        SimpleXML xml2(data);

        SimpleString strName;
        SimpleString strShortcut;
        SimpleString strPath;
        SimpleString strArguments;
        SimpleString strWorkingDir;
        SimpleString strLargeIcon;
        SimpleString strSmallIcon;
        SimpleString strShowCmd;
        SimpleString strFolderName;
        int iAppType = 1;
        int iDesktopShortcutLocation = 0;

        if (xml2.FindTag(("name"), tag, data) > 0)
        {
            strName = data.GetData();
        }
        if (xml2.FindTag(("shortcut"), tag, data) > 0)
        {
            strShortcut = data.GetData();
        }
        if (xml2.FindTag(("path"), tag, data) > 0)
        {
            strPath = data.GetData();
        }
        if (xml2.FindTag(("arguments"), tag, data) > 0)
        {
            strArguments = data.GetData();
        }
        if (xml2.FindTag(("workingdir"), tag, data) > 0)
        {
            strWorkingDir = data.GetData();
        }
        if (xml2.FindTag(("largeicon"), tag, data) > 0)
        {
            strLargeIcon = data.GetData();

		char* fileName = tempFile.CreateFile("AppPortal", "app");
		FILE * fSelfTempFile = fopen(fileName, "w");
		if (fSelfTempFile)
		{
			fprintf(fSelfTempFile, "%s",(char*)strLargeIcon);
			strLargeIcon = fileName;
			fclose(fSelfTempFile);
		}
		else
			strLargeIcon = "noicon";
		//delete [] fileName;
        }
        if (xml2.FindTag(("smallicon"), tag, data) > 0)
        {
            strSmallIcon = data.GetData();

		char* fileName = tempFile.CreateFile("AppPortal", "app");
		FILE * fSelfTempFile = fopen(fileName, "w");
		if (fSelfTempFile)
		{
			fprintf(fSelfTempFile, "%s",(char*)strSmallIcon);
			//delete [] strSmallIcon;
			strSmallIcon = fileName;
			fclose(fSelfTempFile);
		}
		else
			strSmallIcon = "noicon";
		//delete [] fileName;
        }
        if (xml2.FindTag(("showcmd"), tag, data) > 0)
        {
            strShowCmd = data.GetData();
        }
        if (xml2.FindTag(("foldername"), tag, data) > 0)
        {
            strFolderName = data.GetData();
        }
        if (xml2.FindTag(("apptype"), tag, data) > 0)
        {
            iAppType = strtol(data.GetData(), 0, 10);
        }
        if (xml2.FindTag(("desktopshortcutlocation"), tag, data) > 0)
        {
            iDesktopShortcutLocation = strtol(data.GetData(), 0, 10);
        }

        res.Append("Name=%s\n"
            "ShortCut=%s\n"
            "Path=%s\n"
            "Arguments=%s\n"
            "WorkingDir=%s\n"
            "ShowCmd=%d\n"
            "FolderName=%s\n"
            "LargeIcon=%s\n"
            "SmallIcon=%s\n"
            "AppType=%d\n"
            "\n",
            (char*)strName,
            (char*)strShortcut,
            (char*)strPath,
            (char*)strArguments,
            (char*)strWorkingDir,
            (strShowCmd.Length() != 0 ? strtol((char*)strShowCmd, 0, 10) : 0),
            (char*)strFolderName,
	    (char*)strLargeIcon,
	    (char*)strSmallIcon,
            iAppType);
    }

    return dwErrorCode;
}

int GetServerList(char* server, char *proxy, char* cookie, SimpleString& res)
{
    int ret;
    SimpleString strResponse;
    SimpleString buffer;

    buffer.Format(
        "<getfarmproperties-req><cookie>%s</cookie></getfarmproperties-req>",
        cookie);

    // Send the request.
    ret = SendRequest(server, proxy, buffer, strResponse);
    if (ret != 0)
    {
        return ret;
    }

    // Parse the response.
    int dwErrorCode = 0;
    SimpleString strErrorText;
    SimpleString strFarmName;
    SimpleXML xml, tag, data;
    int start = 0;

    xml.LoadData(strResponse);
    if (xml.FindTag("getfarmproperties-rsp", tag, data) <= 0)
    {
        return BROKER_ERROR;
    }

    xml.Copy(data);

    if (xml.FindTag(("errorcode"), tag, data) > 0)
    {
        dwErrorCode = strtol(data.GetData(), 0, 10);
    }
    if (xml.FindTag(("errortext"), tag, data) > 0)
    {
        strErrorText = data.GetData();
    }
    if (dwErrorCode)
    {
        res = (char*)strErrorText;
        return dwErrorCode;
    }

    if (xml.FindTag(("farmname"), tag, data) > 0)
    {
        strFarmName = data.GetData();
    }
    while ((start = xml.FindTag(("server"), tag, data, start)) > 0)
    {
        SimpleXML data2;
        SimpleString strName;
        SimpleString strIPAddress;
        if (data.FindTag(("name"), tag, data2) > 0)
        {
            strName = data2.GetData();
        }
        if (data.FindTag(("ipaddress"), tag, data2) > 0)
        {
            strIPAddress = data2.GetData();
        }

        res.Append("%s,%s\n", (char*)strName, (char*)strIPAddress);
    }

    return dwErrorCode;
}

int GetLaunchInfo(char* server, char *proxy, char* cookie, char* applicationname, SimpleString& res)
{
    int ret;
    SimpleString buffer;
    SimpleString strResponse;
    FILE *ofile;

    buffer.Format(
        "<launchapplication-req>"
            "<cookie>%s</cookie><appname>%s</appname>"
        "</launchapplication-req>",
        cookie, applicationname);


    // Send the request.
    ret = SendRequest(server, proxy, buffer, strResponse);
    if (ret != 0)
    {
        return ret;
    }

    // Parse the response.
    int dwErrorCode = 0;
    char strCommandLine[2048];
    unsigned char hexBuf[4096];
    SimpleString strConnectingText;
    SimpleString strErrorText;
    SimpleString strServerName;
    SimpleString strServerAddr;
    SimpleString strShortcut;
    SimpleString strPath;
    SimpleString strArguments;
    SimpleString strWorkingDir;
    SimpleString strTicket;
    int iShowCmd;
    int iSessionSharing = 1;
    int iAppType = 1;
    strCommandLine[0] = 0;

    SimpleXML xml(strResponse), tag, data;

    memset ( hexBuf, 0x0, 4096 );

//    syslog(LOG_NOTICE, "Request:%s", (char*)buffer);
//    syslog(LOG_NOTICE, "Response:%s", (char*)strResponse);

    if (xml.FindTag(("launchapplication-rsp"), tag, data) <= 0)
    {
        return BROKER_ERROR;
    }

    xml.Copy(data);
    if (xml.FindTag(("errorcode"), tag, data) > 0)
    {
        dwErrorCode = strtol(data.GetData(), 0, 10);
    }
    if (xml.FindTag(("errortext"), tag, data) > 0)
    {
        strErrorText = data.GetData();
    }
    if (xml.FindTag(("servername"), tag, data) > 0)
    {
        strServerName = data.GetData();
    }
    if (xml.FindTag(("serveraddr"), tag, data) > 0)
    {
        strServerAddr = data.GetData();
    }
    if (xml.FindTag(("shortcut"), tag, data) > 0)
    {
        strShortcut = data.GetData();
    }
    if (xml.FindTag(("path"), tag, data) > 0)
    {
        strPath = data.GetData();
    }
    if (xml.FindTag(("arguments"), tag, data) > 0)
    {
        strArguments = data.GetData();
    }
    if (xml.FindTag(("workingdir"), tag, data) > 0)
    {
        strWorkingDir = data.GetData();
    }
    if (xml.FindTag(("showcmd"), tag, data) > 0)
    {
        iShowCmd = strtol(data.GetData(), 0, 10);
    }
    if (xml.FindTag(("sessionsharing"), tag, data) > 0)
    {
        iSessionSharing = strtol(data.GetData(), 0, 10);
    }
    if (xml.FindTag(("apptype"), tag, data) > 0)
    {
        iAppType = strtol(data.GetData(), 0, 10);
        syslog( LOG_NOTICE, "iAppType:%i", iAppType );
    }
    if (xml.FindTag(("ticket"), tag, data) > 0)
    {
        strTicket = data.GetData();

	    // DCT: save the ticket data into a tgt file
        HexToBin( hexBuf, sizeof( hexBuf ), strTicket, strlen( (char *)strTicket) );

        ofile = fopen( Q_SECURE_TICKET_FILE, "wb" );

        if ( ofile == NULL )
        {
            syslog( LOG_ERR, "Unable to write ticket file!" );
        }
        else
        {
            fwrite(( char * )strTicket, 1, strnlen( (char *)strTicket, 4096), ofile );
	        fclose( ofile );
        }
    }

    // Handle the response.
    if (dwErrorCode != 0)
    {
        res = (char*)strErrorText;
        return dwErrorCode;
    }

    // Get display settings.
    switch (iAppType)
    {
    default:
        // default to application

    case 1:  // application
        {
            // Construct the command line.
            if (strArguments.Length() == 0)
            {
                sprintf(strCommandLine, ("%s"), (char*)strPath);
            }
            else
            {
                sprintf(strCommandLine, ("%s %s"), (char*)strPath, (char*)strArguments);
            }

            // Construct a string that gets displayed while connecting to server.
            if (strShortcut.Length() == 0)
            {
                strConnectingText = applicationname;
            }
            else
            {
                strConnectingText = (char*)strShortcut;
            }

            break;
        }

    case 2:  // desktop
        {
            // Construct a string that gets displayed while connecting to server.
            strConnectingText = applicationname;

            break;
        }

    case 3:  // content server
        {
            // Construct the command line.
            strcpy(strCommandLine, strPath);

            // Construct a string that gets displayed while connecting to server.
            strConnectingText = applicationname;

            break;
        }

    case 4:  // content client
        {
            // Construct the command line.
            sprintf(strCommandLine, "%s", (char*)strPath);
            break;
        }
    }

    // Create the launch file.
    res.Append(
        "servername:s:%s\n"
        "serveraddr:s:%s\n"
        "connectingtext:s:%s\n"
        "alternate shell:s:%s\n"
        "shell working directory:s:%s\n"
        "apptype:i:%d\n",
        (char*)(strServerName),
        (char*)(strServerAddr),
        (char*)(strConnectingText),
        (char*)(strCommandLine),
        (char*)(strWorkingDir),
        iAppType);

    // DCT: Append the ticket if one is given by the broker
    if ( strlen( (char *)strTicket ) > 0 )
    {
        syslog(LOG_NOTICE, "Appending ticket data");
        res.Append("ticket:s:%s\n", ( char * )( strTicket ));
    }

    return dwErrorCode;
}

int GetLeastBusyServer(char* server, char *proxy, char* cookie, SimpleString& res)
{
    int ret;
    SimpleString buffer;
    SimpleString strResponse;
    int dwErrorCode = 0;
    SimpleString strErrorText;
    SimpleString strServerName;
    SimpleString strServerAddr;
    SimpleXML xml, tag, data;

    // Generate the request.
    buffer.Format("<getleastbusyserver-req>"
            "<usealternateipaddress>0</usealternateipaddress>"
        "</getleastbusyserver-req>");

    // If there is an active server...
    ret = SendRequest(server, proxy, buffer, strResponse);
    if (ret != 0)
    {
        return ret;
    }

    // Parse the response.
    xml.LoadData(strResponse);
    if (xml.FindTag(("getleastbusyserver-rsp"), tag, data) <= 0)
    {
        return BROKER_ERROR;
    }

    xml.Copy(data);

    if (xml.FindTag(("errorcode"), tag, data) > 0)
    {
        dwErrorCode = strtol(data.GetData(), 0, 10);
    }
    if (xml.FindTag(("errortext"), tag, data) > 0)
    {
        strErrorText = data.GetData();
    }
    if (xml.FindTag(("servername"), tag, data) > 0)
    {
        strServerName = data.GetData();
    }
    if (xml.FindTag(("serveraddr"), tag, data) > 0)
    {
        strServerAddr = data.GetData();
    }

    // Handle the response.
    if (dwErrorCode != 0)
    {
        res = (char*)strErrorText;
        return dwErrorCode;
    }

    res.Format("%s,%s", (char*)strServerName, (char*)strServerAddr);
    return 0;
}

int ChangePassword(char *server, char *username, char *domain, char *oldPassword,
                   char *newPassword, char *cookie, SimpleString & res)
{
    SimpleString buffer;
    SimpleString strResponse;

    // Initialize request
    buffer.Format("p%s,%s,%s,%s", username, domain,
                  oldPassword, newPassword);

    syslog( LOG_NOTICE, "Password change request received");

    // Sending request
    int result = SendRequest(server, "", buffer, strResponse);
    // Log notice
    // syslog(LOG_NOTICE, "PM Request:%s", (char *) buffer);
    // syslog(LOG_NOTICE, "PM Response:%s", (char *) strResponse);

    syslog( LOG_NOTICE, "Password change response received from broker (%i): %s", result, (const char *)strResponse);


//    if (result == 0)
//        return result;
    
    int code;

    char *response = (char *) strResponse;
    char *pch = strchr(response, ',');
    if (pch == NULL)
    {
	if (*response == 0)
        {
            const int errCodeCompNotResponse = 42003;
            res.Format("%s", "Cannot connect to Password Management Server.");
            return errCodeCompNotResponse;
        }

        res.Format("%s", response);
        code = result;
    }
    else
    {

        char errorCode[2];
        strncpy(errorCode, pch - 2, 2);
        code = atoi(errorCode);
        int stringSize = strlen(pch + 1) + 1;
        char *responseString = new char[stringSize];
        strcpy(responseString, pch + 1);
        responseString[stringSize] = '\0';
	res.Format("%s", responseString);
        delete responseString;
    }

    return code;
}
